/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: ajax.js 6285 2017-03-30 17:24:13Z shaun $
 */

"use strict";

//! utility function to get an ajax handle (browser agnostic)
/*!
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * @returns mixed the ajax handle for processing OR boolean false on error.
 */
function getAjaxHandle() {
    var page_request; 

    if (window.XMLHttpRequest) {    // mozilla, safari etc.
        // TODO: declare with var - this is making a global.
        page_request = new XMLHttpRequest();
    } else if (window.ActiveXObject) {  // IE
        try {
            page_request = new ActiveXObject("Msxml2.XMLHTTP");
        } catch(e) {
            try {
                page_request = new ActiveXObject("Microsoft.XMLHTTP");
            } catch(e) {
                return false;
            }
        }
    } else {
        return false;
    }
    return page_request;
}

//! common utility function for posting a request.
/*!
 * @param   string  the url to post to
 *                  (should be relative and LOCAL to the site)
 * @param   fnptr   the callback function to run upon success
 *                  (should accept ajax handle as argument)
 * @param   string  the (encoded) data to post.
 * @param   fnptr   the error function callback. Should accept
 *                  a string as an argument.
 *
 * @return void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 */
function ajaxRequest(url, callback, data_string, errorCallback) {

    var page_request = getAjaxHandle();

    if (!page_request) {
        if (errorCallback) {
            errorCallback("Sorry but browser does not support AJAX or JavaScript is disabled");
        }
        return;
    }
    document.body.style.cursor = "wait";
    page_request.onreadystatechange = function() {
        if (page_request.readyState == 4) {
            document.body.style.cursor = 'auto';
            if (page_request.status == 200) {
                callback(page_request);
            } else {
                errorCallback(page_request.responseText);
            }
        }
    }

    // set up the request.
    page_request.open("POST", url, true);
    page_request.setRequestHeader(
        'Content-Type',
        'application/x-www-form-urlencoded; charset=UTF-8'
    );
    page_request.send(data_string);
}
