
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2011-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *
 * $Id: check-and-uncheck.js 10644 2019-11-14 19:27:57Z deiony $
 */

//! utility function to toggle all checkboxes based on the name
/*!
 *
 * @param   object  the original checkbox from which the event was generated.
 * @param   object  the 'form' object in which the checkboxes are contained.
 *
 * NOTE: if there are any checkboxes unchecked, then this will check them all off.
 *       OTHERWISE, it will toggle between all checked or unchecked.
 */
function toggleAllCheckboxes(origin, formobj, namestr) {
    // sanity check
    if (!origin || !formobj) {
        return;
    }

    // grab the elements
    var checks = new Array();
    for (i = 0; i < formobj.elements.length; i++) {
        var evenOdd = formobj.elements[i].closest('.evenRow, .oddRow');

        // check if parent row even or odd was found
        if (evenOdd) {
            // check if row is hidden
            if (evenOdd.style.display == 'none') {
                continue;
            }
        }

        if (formobj.elements[i].type == "checkbox" &&
            formobj.elements[i].name.substring(0, namestr.length) == namestr) {
            checks[checks.length] = formobj.elements[i];
        }
    }
    // sanity check 2
    if (!checks) {
        return;
    }

    // get the length.
    var numchecks = checks.length;

    // just a single element
    if (!numchecks) {
        // means we toggle.
        if (checks.checked) {
            checks.checked = false;
        } else {
            checks.checked = true;
        }
    } else {
        // multiple elements
        var hasUnchecked = false;
        // multiple elements. Check first for any 'unchecked' boxes.
        for (var c = 0; c < numchecks; c++) {
            if (!checks[c].checked) {
                hasUnchecked = true;
            }
        }
        // pass two: check or uncheck them.
        for (var c = 0; c < numchecks; c++) {
            // we have a checked element
            if (checks[c].checked) {
                // we are only willing to uncheck it if there are only checked elements.
                if (!hasUnchecked) {
                    checks[c].checked = false;
                }
            } else {
                // we have a unchecked element
                checks[c].checked = true;
                origin.checked = true;
            }
        }
    }
}

