/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2018-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 */

"use strict"

if (!window.hasOwnProperty('MS')) {
    window.MS = {};
}

MS.getUrlVars = function() {
    var vars = {};
    var parts = window.location.href.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m,key,value) {
        vars[key] = value;
    });
    return vars;
};

//! Helper function that does all the pre-save procedure and has some validity checks
/*!
 *
 * @returns false
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.saveCountryAuth = function() {
    // retrieve all the inputs for the allowed and blocked countries
    var allowedCountries = document.querySelectorAll('#countryAuth input[name="allowed_countries[]"]');
    var blockedCountries = document.querySelectorAll('#countryAuth input[name="blocked_countries[]"]');

    // determine whether we are running on the admin or user interface
    var module = MS.getUrlVars()['module'];

    // choose the appropriate list of country inputs to disable to reduce the
    // amount of information that needs to be POST-ed based on the interface
    if (module === 'feSecuritySettings') {
        // the user interface is constrained to only being able to save
        // country acl lists with the allow wildcard, so we choose to
        // disabled the allowed country inputs
        var countries = allowedCountries;
    } else {
        // the admin interface can save country acl lists with either wildcard,
        // so we choose the country list with the most inputs
        var countries = (allowedCountries.length > blockedCountries.length) ? allowedCountries : blockedCountries;

        // handle edge case where the default server country has not been set
        // and the admin has chosen to block all countries
        if (allowedCountries.length == 0) {
            // pass an additional input to the backend indicating the edge case
            var all_block_input = document.createElement('INPUT');
            all_block_input.setAttribute("type", "hidden");
            all_block_input.setAttribute("name", "all_block");
            all_block_input.setAttribute("value", "1");
            document.getElementById('country_auth_form').appendChild(all_block_input);
        }
    }

    // disabled the chosen list of country inputs
    for (var i = countries.length - 1; i >= 0; i--) {
        countries[i].setAttribute('disabled', 'disabled');
    }

    // trigger the POST request by submitting the form
    document.getElementById('country_auth_form').submit();
};

MS.saveCountryAuthDefault = function() {
    // reduce the amount of POST data to return
    var countries = document.querySelectorAll('#countryAuth input[name*="_countries"]');
    for (var i = countries.length - 1; i >= 0; i--) {
        countries[i].setAttribute('disabled', 'disabled');
    }

    // set the country auth form action input to save default action
    document.getElementById('action').value = "save_default";

    // submit the country auth form
    document.getElementById('country_auth_form').submit();
};

//! Helper method that will register the JS for the multiselect
/*!
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.multiselectRegisterEvent = function(id) {
    var items = document.getElementById(id).getElementsByTagName('LI');
    for (var i = 0; i < items.length; i++) {
        items[i].onclick = MS.multiselectCallback;
    }
    var tooltips = document.getElementById(id).querySelectorAll('.mm-multiselect .tooltip-description');
    for (var i = 0; i < tooltips.length; i++) {
        tooltips[i].onmousemove = MS.showTooltip;
        tooltips[i].onmouseout = MS.clearTooltip;
    }
};


//! Helper method to determine the relative coord of an object relative to a parent
/*!
 * @param   object mouseEvent the mouse event
 * @param   object selector the jQuery selector
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.findObjectCoords = function(mouseEvent, selector) {
    if (selector === undefined || typeof selector !== 'object') {
        return;
    }

    var obj = selector;
    var ret = { x: 0, y: 0};
    var objOffset = { x: 0, y: 0};
    while (obj.offsetParent) {
        objOffset.x += obj.offsetLeft;
        objOffset.y += obj.offsetTop;
        obj = obj.offsetParent;
    }
    if (mouseEvent) {
        //FireFox
        ret.x = mouseEvent.pageX;
        ret.y = mouseEvent.pageY;
    } else {
        //IE
        ret.x = window.event.x + document.body.scrollLeft - 2;
        ret.y = window.event.y + document.body.scrollTop - 2;
    }

    ret.x -= objOffset.x;
    ret.y -= objOffset.y;

    return ret;
}

//! Helper method to show the tooltip using the data-desc attribute
/*!
 * @param   object evt, the event that trigger
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.showTooltip = function(evt) {
    var placeholder = document.getElementById("tooltip-placeholder");
    if (placeholder === null) {
        return;
    }

    var content = evt.target.getAttribute('data-desc');
    if (content === null || content === '') {
        return;
    }

    // This works great if the parent has postion relative
    var pos  = MS.findObjectCoords(evt, evt.target.closest('.country-auth-step2'));
    if (!pos) {
        return;
    }
    var x = pos.x + 20;
    var y = pos.y + 20;

    placeholder.style.top = y + 'px';
    placeholder.style.left = x + 'px';
    placeholder.style.display = 'block';
    placeholder.textContent = content;
};


//! Helper method to clear and hide the tooltip
/*!
 * @param   evt
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.clearTooltip = function(evt) {
    var placeholder = document.getElementById("tooltip-placeholder");
    if (placeholder === null) {
        return;
    }
    placeholder.style.display = "none";
};

//! Callback method for selecting nodes
/*!
 * @param   evt
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.multiselectCallback = function(evt) {
    evt.stopPropagation();
    if (this.classList.contains('mm-optgroup') || this.classList.contains('non-selectable')) {
        return;
    }
    this.classList.toggle('selected');
};

//! Convinience function that will select all allowed nodes from source
/*!
 * @param   sourceSelector
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.multiselectToggleAll = function(evt, sourceId) {
    if (evt.target.textContent === 'Select All') {
        var selector = 'li:not(.mm-optgroup):not(.non-selectable):not(.selected)';
        var label = 'Select None';
    } else {
        var selector = 'li.selected';
        var label = 'Select All';
    }

    var source = document.getElementById(sourceId);
    var items = source.querySelectorAll(selector);
    for (var i = items.length - 1; i >= 0; i--) {
        items[i].classList.toggle('selected');
    }
    evt.target.textContent = label;
};

MS.multiselectReset = function(destination) {
    var buttons = document.getElementsByClassName('minibutton right');
    for (var i = buttons.length - 1; i >= 0; i--) {
        buttons[i].firstElementChild.textContent = 'Select All';
    }
    var items = destination.getElementsByClassName('selected');
    for (var i = items.length - 1; i >= 0; i--) {
        items[i].classList.remove('selected');
    }
};

//! Helper method that will move .selected nodes from source to destination
/*!
 * @param   sourceSelector
 * @param   destinationSelector
 *
 * @returns
 *
 * @author  Rares Harnagea <rares@linuxmagic.com>
 */
MS.multiselectMove = function(sourceId, destinationId) {
    var source = document.getElementById(sourceId);
    var destination = document.getElementById(destinationId);

    var items = source.getElementsByClassName('selected');
    for (var i = items.length - 1; i >= 0; i--) {
        var item = items[i];
        item.classList.remove('selected');
        destination.insertBefore(item, destination.firstChild);
    }

    var opts = destination.querySelectorAll("li:not(.mm-optgroup):not(.non-selectable)");
    var options = Array.prototype.slice.call(opts);
    var arr = options.map(function(i, o) { return { t: i.textContent, v: i.getAttribute('data-value'), c: i.innerHTML } });
    arr.sort(function(o1, o2) { return o1.t > o2.t ? 1 : o1.t < o2.t ? -1 : 0; });

    for (var j = 0; j < options.length; j++) {
       options[j].innerHTML = arr[j].c;
       options[j].setAttribute('data-value', arr[j].v);
       options[j].firstElementChild.name = destinationId + '[]';
    }

    MS.multiselectReset(destination);
};

MS.injectErrorMessage = function(message) {
    var errors = document.getElementById('errors');

    // create and inject new error div if it does not already exist
    if (!errors) {
        errors = document.createElement('div');
        errors.id = 'errors';
        errors.appendChild(document.createElement('ul'));

        var ms_wrapper = document.getElementById('magicspam-wrapper');
        ms_wrapper.insertBefore(errors, ms_wrapper.firstChild);
    }

    // create the error list item
    var error_list_item = document.createElement('li');
    error_list_item.innerText = message;

    // append the error list item to the error list
    var error_list = errors.getElementsByTagName('ul')[0];
    error_list.appendChild(error_list_item);
};

MS.injectNoticeMessage = function(message) {
    var notices = document.getElementById('notices');

    // create and inject new notice div if it does not already exist
    if (!notices) {
        notices = document.createElement('div');
        notices.id = 'notices';
        notices.appendChild(document.createElement('ul'));

        var ms_wrapper = document.getElementById('magicspam-wrapper');
        ms_wrapper.insertBefore(notices, ms_wrapper.firstChild);
    }

    // create the notice list item
    var notice_list_item = document.createElement('li');
    notice_list_item.innerText = message;

    // append the notice list item to the notice list
    var notice_list = notices.getElementsByTagName('ul')[0];
    notice_list.appendChild(notice_list_item);
};

MS.discardChangesCountryAuth = function() {
    // attempt to reload the page via the active module in the menu
    // to avoid potentially resending a previous POST request
    var module = document.querySelector('#menu li[class="active"] a');
    if (module) {
        // will trigger the reload of the page
        window.location.href = module.href;
    } else {
        // fallback to reloading the with previous POST
        location.reload();
    }
};

document.addEventListener('DOMContentLoaded', function() {
    if (!document.getElementById('countryAuth')) {
        return;
    }
    MS.multiselectRegisterEvent('allowed_countries');
    MS.multiselectRegisterEvent('blocked_countries');
});
