/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: feedback.js 10938 2020-01-10 21:11:41Z deiony $
 */

// set up a namespace to try and avoid naming collisions
if (!MS) {
    var MS = {};
}

//! update the feedback rating
/*!
 * @param   rating  the selected rating
 */
MS.updateFeedbackRating = function(rating) {
    // turn on up to the selected star
    for (i = 0; i < MS.stars.length; i++) {
        if (i > rating - 1) {
            MS.stars[i].classList.remove("on");
        } else {
            MS.stars[i].classList.add("on");
        }
    }

    // update the value of the rating input
    document.getElementById('feedback-rating-input').value = rating;

    // set focus on the comments textarea
    document.getElementById('feedback-comments-textarea').focus();
}

MS.submitFeedbackForm = function() {
    // ensure that feedback rating has been set
    if (document.getElementById('feedback-rating-input').value == 0) {
        MS.buildNotice(MS.intl('feedback_no_rating_failure'), 'errors');
        return;
    }

    // submit the feedback form
    document.getElementById('feedback-form').submit();
}

document.addEventListener("DOMContentLoaded", function(event) {
    // retrieve and store the list of all feedback rating stars
    MS.stars = document.querySelectorAll('#feedback-rating span[class~="feedback-rating-star"]');

    // set events for showing and hiding faded feedback rating stars
    for (i = 0; i < MS.stars.length; ++i) {
        // set the onmouseenter event to show faded star
        MS.stars[i].onmouseenter = function(e) {
            for (j = 0; j < MS.stars.length; j++) {
                // only show faded star when the star is not ON
                if (!MS.stars[j].classList.contains("on")) {
                    MS.stars[j].classList.add("fade");
                }
                if (MS.stars[j] == e.target) {
                    break;
                }
            }
        }
        // set the onmouseenter event to hide faded star
        MS.stars[i].onmouseleave = function(e) {
            for (j = 0; j < MS.stars.length; j++) {
                MS.stars[j].classList.remove("fade");
            }
        }
    }
});
