/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Anonymous
 *
 * $Id: ip-form.js 10599 2019-11-12 18:02:33Z deiony $
 */

// set up a namespace to try and avoid naming collisions
if (!MS) {
     var MS = {};
}

//! form submitter for adding entries
/*
 * @param string form_id the form id
 * @param string action_id the action id
 * @param string action_val the action value to set
 *
 * @author Anonymous
 */
function formSubmit(form_id, action_val) {
    var form   = document.getElementById(form_id);
    var action = document.forms[form_id].ms_action;
 
    // validate Sender Country Blacklist dropdown menu of countries
    if (form_id == 'country_black_list_form' && action_val == 'add') {
        var country = document.getElementById("country_id");

        if (!country.options[country.selectedIndex].value) {
            MS.buildNotice(MS.intl('invalid_entry'), 'errors');
            // prevent page from refreshing 
            event.preventDefault(); 
            return;
        } 
    } 

    if (!form || !action) {
        // popup an alert error?
        return;
    }
    // Confirms deletion
    if (action_val == 'delete' && !confirm(MS.intl('exemptionsConfirmDelete'))) {
        return;
    }

    action.value = action_val;
    form.submit();
}

//! utility function to clear all checkboxes based on the name
/*!
 * @param   object  the original checkbox from which the event was generated.
 * @param   object  the 'form' object in which the checkboxes are contained.
 *
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function clearAllCheckboxes(origin, formobj, namestr) {
    // sanity check #1
    if (!origin || !formobj) {
        return;
    }

    // grabs the elements of type "checkbox"

    var checks = new Array();
    for (i = 0; i < formobj.elements.length; i++){
        if (formobj.elements[i].type == "checkbox" &&
            formobj.elements[i].name.substring(0, namestr.length) == namestr) {
            checks[checks.length] = formobj.elements[i];
        }
    }

    // sanity check #2
    if (checks.length === 0) {
        return;
    }

    // grabs the number of checkbox elements
    var numchecks = checks.length;

    // just a single element, otherwise multiple elements
    if (!numchecks) {
            checks.checked = false;
    } else {
        for (var c = 0; c < numchecks; c++) {
                checks[c].checked = false;
        }
    }
}

//! clears all the checkboxes and then submits the form
/*!
 * @param string form_id the form id
 * @param string action_val the action value to set
 * @param string chk_name the name of the checkbox
 *
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function checkAndSubmitForm(form_id, action_val, chk_name) {
    // Clears the currently selected checkboxes
    var chkbox = null;
    var currentForm = document.getElementsByName(form_id)[0];
    if (!currentForm) {
        return;
    }
    // Attempts to grab the mastercheckbox to clear it
    chkbox = currentForm.elements[chk_name];
    masterChkbox = currentForm.elements['all'];

    // If no element was selected successfully, then ends
    // Else clears the master checkbox (i.e. the 'all')
    if (!chkbox || !masterChkbox) {
        return;
    }
    clearAllCheckboxes(document, currentForm, 'entry[');
    masterChkbox.checked = false;
    chkbox.checked = true;

    // Submits the form
    formSubmit(form_id, action_val);
}

//! Confirms deletion before settin action on RBL form
/*!
 * @param string listName the list to be deleted
 *
 * @return false
 *
 * @author Daniel Peitl <danielp@linuxmagic.com>
 * @author Robert Bisewski <robertb@linuxmagic.com>
 *
 * TODO: May be merged into a more generic function
 */
function confirmDeleteRBL(listName) {
    // checks if parameter is valid
    if (listName === undefined ) {
        return false;
    }
    if (!confirm(MS.intl('rblConfirmDelete') + ' ' + decodeURIComponent(listName) + '?')) {
        return false;
    }
    var fields = {
        'hiddenFields': {
            'module': 'settings',
            'page': 'ip_reputation',
            'submodule': 'rblLists',
            'delete': listName,
            'ms_action': 'delete_rbl'
        },
        'action':'#rbl'
    };

    doFormSubmit(fields);
    // we should never reach the next line....
    return false;
}
