/*
 * Simple set of function(s) used by the 'first login' set password system.
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2013-2019 All Rights Reserved.
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id$
 */

//! utility function for checking if the password fields match up and meet requirements
/*!
 * NOTE: right now this only validates length check (5 chars). This *should* do an ajax
 *       callback for proper strength validation - but its better than what previously existed.
 */
function checkPasswordFields(invalidMessage, lengthMessage) {
    // grab our elements
    var passwordField = document.getElementById('mmc_new_password');
    var confirmField = document.getElementById('mmc_confirm_password');
    var statusMessage = document.getElementById('statusMessage');
    var loginSave = document.getElementById('loginSave');

    // sanity check
    if (!passwordField || !confirmField || !statusMessage || !loginSave) {
        return;
    }
    // grab the field values
    var pwdVal = passwordField.value;
    var confirmVal = confirmField.value;

    // if they don't match, then...
    if (pwdVal != confirmVal) {
        statusMessage.innerHTML = invalidMessage;
        loginSave.disabled = "disabled";
    // otherwise, test the strength...
    } else if (pwdVal.length < 5) {
        statusMessage.innerHTML = lengthMessage;
        loginSave.disabled = "disabled";
    // otherwise we're good to go.
    } else {
        statusMessage.innerHTML = '';
        loginSave.disabled = "";
    }
}

function checkLogin() {
    var loginField = document.getElementById('mmc_login');
    var passwordField = document.getElementById('mmc_password');
    var loginButton = document.getElementById('loginSubmit');
    
    if (!loginField || !passwordField || !loginButton) {
        return;
    }
    var pwdval = passwordField.value;
    var loginval = loginField.value;

    if (loginval.length && pwdval.length) {
        loginButton.disabled = "";
    } else {
        loginButton.disabled = "disabled";
    }
}

//! minor callback function to set the focus of the password field.
function setFieldFocus() {
    var passwordField = document.getElementById('mmc_new_password');
    var loginField = document.getElementById('mmc_login');
    if (passwordField) {
        passwordField.focus();
    }
    if (loginField) {
        loginField.focus();
    }

    // in case of autofill, enable the login button
    checkLogin();
}

// register the 'onload' event to set the starting focus.
addLoadEvent(setFieldFocus);
