/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: magicspam_menu.js 3825 2014-09-29 21:32:38Z shaun $
 */

if (!MS) {
    var MS = {};
}

//! debug mode on or off.
MS.debug = false;

//! Extract the domain from a given email address 
/*
 * @params  String  email address
 *
 * @return  String  domain of email address
 */
MS.getDomain = function(email) {

    // Basic validation
    if (!email) {
        return;
    }

    // Break up our email address
    var email_pieces = email.split("@");
    
    // Sanity check, make sure we got something back
    if (!email_pieces || !email_pieces[1]) {
        return;
    }

    // Attempt to extract the domain from the email address
    var address_domain = email_pieces[1];

    // Return the domain of our email address 
    return address_domain;
}

MS.buildMenuLink = function(fname, fargs, array, desc) {
    // sanity check
    if (!fname || !fargs || !desc) {
        return;
    }

    // convert arguments array into arguments string
    var args = fargs.map(function(arg) {
        return "'" + arg + "'";
    }).join(',');

    // surround with square brackets when argument should be an array
    if (array) {
        args = '[' + args + ']';
    }

    // build the onclick function for the link
    var func = fname + '(' +  args + ')';

    // build and return the link
    return '<a href="#" onclick="' + func + '; return false;">' + desc + '</a>';
}

//! Creates a GUI menu which allows you to select various options 
/*
 * @param  element  HTML given via the log search
 * @param  type     Mail type (e.g. HAM, SPAM, EXEMPT, RATELIMIT, etc)
 * @param  column   Column to display the link in (e.g. IP, Sender, etc)
 * @param  optional Optional information
 */
MS.menuGUI = function(element, type, column, optional) {
    // sanity check
    if (!element || !type || !column || !document.body) {
        return;
    }

    // ensure that the column is Type, IP, Country or Sender
    if (column !== "Type" && column !== "IP" && column !== "Country" && column !== "Sender") {
        return;
    }

    // retrieve the coordinates and content of the element
    var coords = MS.getCoords(element);
    var elementContent = element.innerHTML;
    if (!coords || !coords.x || !coords.y) {
        return;
    }

    // close any previously opened instance of the menu
    if (document.getElementById("MS_logsearch_menu")) {
        MS.closeMenu("MS_logsearch_menu");
    } 

    // initialize a new DIV for the log search menu
    var menu = document.createElement("DIV");
    menu.id = "MS_logsearch_menu";
    menu.className = "menuGUI";
    menu.style.left = coords.x + "px";
    menu.style.top = coords.y + "px";
    menu.innerHTML = "";

    // add link to blacklist country of ham email
    if (column === 'Country' && type === 'ham') {
        menu.innerHTML += MS.buildMenuLink('logSearchAddCountryToList', [elementContent, 'black', MS.intl('ls_alert_blacklist'), optional], false, MS.intl('ls_block_Country'));
    // add link to replay ham email
    } else if (column === 'Type' && type === 'ham' && optional > -1) {
        menu.innerHTML += MS.buildMenuLink('logSearchRunReplay', MS.searchResult[optional], true, MS.intl('ls_block_message'));
    // add link to whitelist ratelimited user
    } else if (column == "Sender" && type === "ratelimit") {
        menu.innerHTML += MS.buildMenuLink('logSearchAddIPToList', [elementContent, 'white_ratelimit_outbound', MS.intl('ls_alert_whitelist')], false, MS.intl('ls_allow_user_ratelimit'));
    // add link to blacklist sender of ham email
    } else if (type === 'ham') {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'black', MS.intl('ls_alert_blacklist')], false, MS.intl('ls_block_' + column));
    // add link to whitelist ip and sender
    } else if (type === 'spam') {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'white', MS.intl('ls_alert_whitelist')], false, MS.intl('ls_allow_' + column));
        // additional add link to whitelist sender domain
        if (column === 'Sender') {
            menu.innerHTML += MS.buildMenuLink('logSearchAddSenderToList', [MS.getDomain(elementContent), 'white_domain', MS.intl('ls_alert_whitelist_domain')], false, MS.intl('ls_allow_SenderDomain'));
        }
    // add link to remove ip whitelist
    } else if (type === "exempt_ip_whitelist") {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'remove_white', MS.intl('ls_alert_remove_whitelist')], false, MS.intl('ls_remove_white_ip'));
    // add link to remove ip blacklist
    } else if (type === "spam_ip_blacklist") {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'remove_black', MS.intl('ls_alert_remove_blacklist')], false, MS.intl('ls_remove_black_ip'));
    // add link to whitelist ratelimited ip
    } else if (type === "ratelimit_inbound") {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'white_ratelimit_inbound', MS.intl('ls_alert_whitelist')], false, MS.intl('ls_allow_ip_ratelimit'));
    // add link to whitelist or blacklist ip when ratelimited
    } else {
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'white', MS.intl('ls_alert_whitelist')], false, MS.intl('ls_allow_IP'));
        menu.innerHTML += MS.buildMenuLink('logSearchAdd' + column + 'ToList', [elementContent, 'black', MS.intl('ls_alert_blacklist')], false, MS.intl('ls_block_IP'));
    }

    // add link to close the menu
    menu.innerHTML += MS.buildMenuLink('MS.closeMenu', ['MS_logsearch_menu'], false, MS.intl('ls_cancel'));
 
    // append the menu which has the coordinates of the element
    document.body.appendChild(menu);
}

//! Grabs the coordinates of a given element
MS.getCoords = function(element) {
 
    // Sanity check, gotta make sure we have something
    if (!element) {
        return;
    } 

    // vars used to store x any y coords
    var lx, ly;

    // Cycle thru the page and figure out where we are
    for (lx = 0, ly = 0; element != null; element = element.offsetParent) {
        lx += element.offsetLeft;
        ly += element.offsetTop;
    }

    // Adjusts the coords so they appear under the link
    lx -= 2;
    ly += 15;

    // Validation to make sure that lx is > 0, we don't want elements falling off the browser
    if (lx < 0) {
        lx = 0;
    }

    // Returns an array of x/y coords 
    return {x: lx, y: ly};
}

//! Closes the menu if the X is clicked 
MS.closeMenu = function(id) {

    // Basic validation
    if (!id) {
        return;
    }

    // Attempt to grab the element and its parent 
    var elmToDelete = document.getElementById(id);
    var elmParent = elmToDelete.parentNode;

    // If one or the other don't exist, just return
    if (!elmToDelete || !elmParent) {
        return;
    }

    // Grabs the element and removes it
    elmParent.removeChild(elmToDelete);
}
