/*!
 * @param string name The rule name (internal, e.g. block_lists)
 *
 * @return mixed Object, or undefined if not found.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * a rule is associated with a slider span... find it.
 *
 * we do this right now by finding the rule's input, and then
 * finding its sibling slider span.
 * the HTML currently is:
 * <div class="centerColumn">
 *   <div class="slider-frame">
 *     <span class="slider-button" onclick..."></span>
 *   </div>
 *   <input> <!-- the rule's input. -->
 * </div>
 */
MS.findRuleSlider = function(name) {
    if (name === undefined) {
        this.log("findRuleSlider: invalid parameter");
        return undefined;
    }
    // we need the rule input first.
    var input = this.findRuleInput(name);
    if (!input) {
        this.log("findRuleSlider: rule input not found");
        return undefined;
    }
    // its container <div>
    var parentDiv = input.parentElement;
    if (!parentDiv) {
        this.log("findRuleSlider: rule input parent not found");
        return undefined;
    }
    // it should have a slider frame div, the rule input, and the 'original value' input
    if (parentDiv.children.length !== 3) {
        this.log("findRuleSlider: unexpected parent child count: "
            + parentDiv.children.length);
        return undefined;
    }
    // get the slider frame div.
    var sliderFrame = parentDiv.children[0];
    // it should have the slider button span.
    if (sliderFrame.children.length !== 3) {
        this.log("findRuleSlider: unexpected frame child count: "
            + sliderFrame.children.length);
        return undefined;
    }
    // and get the slider button span!
    var slider = sliderFrame.children[0];
    return slider;
};

