/* 
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2011-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * Contains an invaluable utility function to alter the action of a form,
 * with enough validation to prevent horrible accidents.
 *
 * CAUTION: This function was carefully designed to work with every browser
 * supported by MagicSpam, including the frustrating IE7. Please don't touch
 * anything unless you also massively crossbrowser test.
 *
 * $Id: set-action.js 4221 2014-11-04 23:28:57Z shaun $
 */

function setAction(form_id, action_val) {
    
    // Basic validation
    if (!form_id || !action_val) {
        MS.log('setAction: Insufficient input!');
        return;
    }

    // Make certain inputs are proper strings
    // It's a good idea to have extra validation in this function
    // since we mess around with forms... 
    if (typeof form_id !== 'string' || typeof action_val !== 'string' 
        || form_id.length === 0 || action_val.length === 0) {
        MS.log('setAction: Invalid parameter!');
        return;
    }
   
    // Attempt to grab the form and its ms_action
    var form   = document.getElementsByName(form_id)[0];
    var action = document.forms[form_id].ms_action;

    // Check if we've obtained a valid form
    if (!form) {
        MS.log('setAction: form not found with id: ' + form_id);
        return;
    }

    // Check if we've obtained a valid action 
    if (!action) {
        MS.log('setAction: ms_action not present in: ' + form_id);
        return;
    }

    // Otherwise all is well, so set the action value 
    action.value = action_val;
}
