<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.register.php 10912 2020-01-08 01:25:07Z deiony $
 */

include_once('lib.cmdwrapper.php');

class Register extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    //! class constructor
    /*!
     * NOTE: we want this class type to be a singleton, so we
     * deny access to the constructor
     * @author Sean West <sean@linuxmagic.com>
     *         Shaun Johnson <shaun@linuxmagic.com>
     */
    protected function __construct() {
        // instantiate back to the parent.
        parent::__construct(REGISTER);
    }

    //! static function to get the current singleton instance
    /*!
     * @returns LicenseManager  the active singleton instance.
     *
     * If the instance is not yet created, then this will create it.
     * @author Sean West <sean@linuxmagic.com>
     *         Shaun Johnson <shaun@linuxmagic.com>
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new Register;
        }
        return self::$object;
    }

    //! access function to update the license with a optional new license key.
    /*!
     * @returns boolean success or fail
     * @author Sean West <sean@linuxmagic.com>
     *         Shaun Johnson <shaun@linuxmagic.com>
     *
     * Errors will retain a detailed error message that can be fetched by
     * a call to the 'error()' function.
     */
    public function toggle() {
        // get the current status.
        $status = getControl("enabled", 0);
        if ($status == 0) {
            $args = array('-a');
        } else {
            $args = array('-d');
        }
        $buffer = "";
        // attempt to run the execution
        $rv = $this->execRead($args, $buffer);
        if ($rv !== 0 || strlen($buffer) > 0) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }
}
