<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.rejections.php 5666 2016-06-27 17:28:19Z shaun $
 *
 * This is a container to access the rate limiter 'manager' utility.
 * There are only two operations available in this class:
 *
 *     list
 *     remove
 */
include_once('lib.cmdwrapper.php');

class RejectionsManager extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    //! class constructor
    /*!
     * @author Shaun Johnson <shaun@linuxmagic.com>
     * we want this class type to be a singleton, so we deny access to the constructor
     *
     */
    protected function __construct() {
        // instantiate back to the parent.
        parent::__construct(REJECTIONS_MANAGER);
    }

    //! static function to get the current singleton instance
    /*!
     * @returns RateLimiterManager  the active singleton instance.
     *
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new RejectionsManager;
        }
        return self::$object;
    }

    //! function to 'remove' a rejection message
    /*!
     * @param   string      the entry to remove from the blocked list.
     *
     * @returns boolean     success or fail of operation.
     */
    public function removeRejection($entry) {
        $buffer = "";
        // basic sanity check
        if (!is_string($entry) || !strlen($entry)) {
            $this->lastError = "Invalid parameter";
            return FALSE;
        }
        $args = array('-r', '-n');
        $args[] = $entry;
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }

    //! function to 'save' a rejection message
    /*!
     * @param   string  the 'name' of the rule to set the message for
     * @param   string  the message to set
     *
     * @returns boolean success or fail of operation.
     */
    public function setRejection($name, $message) {
        $buffer = "";
        // basic sanity check
        if (  !is_string($name)
           || !strlen(trim($name))
           || !is_string($message)
           || !strlen(trim($message)) ) {
            $this->lastError = "Invalid parameter";
            return FALSE;
        }
        $args = array('-a', '-n', trim($name), '-m', trim($message));
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }
}
