<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id:
 */
include_once('lib.cmdwrapper.php');

class SpamAssassinStatsUtility extends cmdWrapper {
    // internal protected singleton.
    protected static $object = NULL;

    //! static function to get the current singleton instance
    /*!
     * @returns SpamAssassinStatsUtility  the active singleton instance
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new self(SA_STATS);
        }
        return self::$object;
    }

    //! runs magicspam SA stats utility
    /*!
     * @param    numeric   the number of log days to look back
     * @param    numeric   the number of top SA rules to retrieve
     * @param    string    the buffer for the results of the retrieval
     *
     * @returns  boolean   whether the call to the SA stats utility was successful
     */
    public function callSpamAssassinStatsUtility($days, $rules, &$buffer) {
        // sanity check
        if (empty($days) || !is_numeric($days) ||
            empty($rules) || !is_numeric($rules)) {
            $this->lastError = "Invalid arguments";
            return FALSE;
        }
       
        // build the list of arguments
        $args = array(
            '-d',
            $days,
            '-r',
            $rules,
        );

        // execute the SA stats utility with the arguments
        // the SA stats utility is expected to output a json encoded string
        $rv = $this->execRead($args, $buffer);

        // ensure that the execution was successfuly and we got the string
        if ($rv !== 0 || !strlen($buffer)) {
            $this->lastError = "Failed to retrieve SA stats results";
            return FALSE;
        }

        // the SA stats utility has been called successfully
        return TRUE;
    }
}

