<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_db_pear_sth.php 151 2017-03-02 01:08:59Z rares $
 *
 * This is an abstract DB class that follows PEAR syntax
 */


require_once(__DIR__ . '/interface.wiz_db_abstract.php');
require_once(__DIR__ . '/lib.wizard_debug.php');

class WizDbPearSth extends WizardDebug implements WizDbSth {

    //! @object with the result set
    protected $_sth;

    //! The constructor of our class
    /*!
     * @param [mixed] int if this is a prepared statement or object if is a result set
     *
     * @return void
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function __construct($sth = NULL) {
        parent::__construct();
        // $sth could 
        $this->_sth = $sth;
    }

    //! Helper method to translate the fetch method to PEAR
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function _translateFetchMethod($mode = '') {
        if (!isset($mode) || empty($mode) || $mode === self::FETCH_NUM) {
            // these are PEAR constants
            return DB_FETCHMODE_ORDERED;
        } elseif ($mode === self::FETCH_ASSOC || $mode === DB_FETCHMODE_ASSOC) {
            return DB_FETCHMODE_ASSOC;
        }

        // in the worst case return fetch ordered
        return DB_FETCHMODE_ORDERED;
    }

    //! Method used to fetchRow records from a results set
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function &fetchRow($mode = 0) {
        $null = NULL;
        if (!isset($this->_sth) || !is_a($this->_sth, 'DB_result')) {
            return $null;
        }
        $ret = $this->_sth->fetchRow($this->_translateFetchMethod($mode));
        return $ret;
    }

    //! Method used to fetch alias for fetchRow records from a results set
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function &fetch($mode = 0) {
        // alias for fetchRow
        return $this->fetchRow($mode);
    }


    //! Method used to return the error message if there was an error
    /*!
     * @return mixed string with the message or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getMessage() {
        return $this->_sth->getMessage();
    }


    //! Method used to get the number of rows from a SELECT statement
    /*!
     * @return mixed int with the number or rows from the last query or NULL id there was a problem
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function numRows() {
        $null = NULL;
        if (!isset($this->_sth)) {
            return $null;
        }
        return $this->_sth->numRows();
    }

    //! Method used to free the memory with the last statement
    /*!
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function free() {
        return $this->_sth->free();
    }

    //! Helper method to free the memory with the results
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function freeResult($result) {
        if (isset($result)) {
            unset($result);
        }
     }

    //! Getter method to return the acctualy DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getSth() {
        return $this->_sth;
    }


}
