<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2016-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 *
 * Note: The starting point of this library was: https://github.com/PHPMailer/PHPMailer
 * class.phpmailer.php
 * 
 * Usage:
 * $mail =  new WizardMail();
 * // what mail server do you want to use
 * $mail->setHost('mail.cityemail.com');
 * // if there are problem then set the debug level
 * //$mail->setDebugLevel(4);
 * 
 * // set the Port, by default is 25
 * $mail->setPort(587);
 * // If you want to use TLS then set this option
 * $mail->setTls(TRUE);
 * 
 * // If you want to send authenticated emails then set the username and password 
 * // for the SMTP Authentication
 * $mail->setUsername('username');
 * $mail->setPassword('password');
 *
 * // If you want to set CID Type and Hash
 * $mail->setCidType('COOKIE');
 * $mail->setCid('MP-5b108553dd9f5');
 * 
 * Set the Sender Mail From
 * $mail->setSender('email_address');
 * 
 * $mail->addTO('email_address1'[, 'friendly_name1']);
 * // call the above as many time you need
 * $mail->addTO('email_address1'[, 'friendly_name1']);
 * $mail->addTO('email_address2'[, 'friendly_name2']);
 * 
 * $mail->addCC('email_address2'[, 'friendly_name2']);
 * // call the above as many time you need 
 * 
 * $mail->addBCC('email_address2'[, 'friendly_name2']);
 * // call the above as many time you need 
 *
 * $mail->addReplyTo('email_address2'[, 'friendly_name2']);
 * // call the above as many time you need 
 * 
 * $mail->addAttachment('path_to_file', 'name_of_the_attachement');
 * // call the above as many time you need 
 *
 * $mail->setSubject('This is the subject of my email');
 *
 * $mail->setBody("Add a HTML or a Plain Text body");
 * 
 * $mail->setAltBody('If you added a HTML body and you want to plain text body for clients that do not support HTML');
 * 
 * if ($mail->send()) {
 *     print "Email sent\n";
 * }
 * 
 * // There are also functions to set Priority, confirmation, custom headers and many more
 * 
 * // For testing you can use the flag pretend if you do not want to send the actual email
 * 
 */

class WizardMail extends WizardSmtp {
    
    const QMAIL_INJECT_BIN = '/usr/sbin/qmail-inject';

    // Flag to determine the qmail caller 
    protected $_qmail_inject_caller = NULL;

    //! Email priority.
    /*!
     * Options: null (default), 1 = High, 3 = Normal, 5 = low.
     * When null, the header is not set at all.
     * @var integer
     */
    protected $_priority = NULL;

    //! The character set of the message.
    /*!
     * @var string
     */
    protected $_charSet = 'utf-8';

    //! The MIME Content-type of the message.
    /*!
     * @var string
     */
    protected $_contentType = 'text/plain';

    //! The message encoding.
    /*!
     * Options: "8bit", "7bit", "binary", "base64", and "quoted-printable".
     * @var string
     *
     * XXX: WHY IS THIS PUBLIC?
     */
    public $_encoding = '8bit';

    //! The Sender email (Return-Path) of the message.
    /*!
     * If not empty, will be sent via -f to sendmail or as 'MAIL FROM' in smtp mode.
     * @var string
     */
    protected $_sender = '';

    //! the sender friendly name
    protected $_senderName = '';

    //! The username if you want to use SMTP authentication
    //  @string 
    protected $_username = '';

    //! The password if you want to use SMTP authentication
    //  @string 
    protected $_password = '';

    //! the preferred line ending.. ? odd.
    protected $LE = "\n";

    //! The Return-Path of the message.
    /*!
     * If empty, it will be set to either From or Sender.
     * @var string
     * @deprecated Email senders should never set a return-path header;
     * it's the receiver's job (RFC5321 section 4.4), so this no longer does anything.
     * @link https://tools.ietf.org/html/rfc5321#section-4.4 RFC5321 reference
     */
    protected $_returnPath = '';

    //! The Subject of the message.
    /*!
     * @var string
     */
    protected $_subject = '';

    //! An HTML or plain text message body.
    /*!
     * If HTML then call isHTML(true).
     * @var string
     */
    protected $_body = '';

    //! The plain-text message body.
    /*!
     * This body can be read by mail clients that do not have HTML email
     * capability such as mutt & Eudora.
     * Clients that can read HTML will view the normal Body.
     * @var string
     */
    protected $_altBody = '';

    //! The complete compiled MIME message body.
    /*!
     * @var string
     */
    protected $_MIMEBody = '';

    //! The complete compiled MIME message headers.
    /*!
     * @var string
     */
    protected $_MIMEHeader = '';

    //! Extra headers that createHeader() doesn't fold in.
    /*!
     * @var string
     */
    protected $_mailHeader = '';

    //! Word-wrap the message body to this number of chars.
    /*!
     * Set to 0 to not wrap. A useful value here is 78, for RFC2822 section 2.1.1 compliance.
     * @var integer
     */
    protected $_wordWrap = 0;

    //! An ID to be used in the Message-ID header.
    /*!
     * If empty, a unique id will be generated.
     * You can set your own, but it must be in the format "<id@domain>",
     * as defined in RFC5322 section 3.6.4 or it will be ignored.
     * @see https://tools.ietf.org/html/rfc5322#section-3.6.4
     * @var string
     */
    protected $_messageID = '';

    //! This is the message ID that is going to be added
    /*!
    */
    protected $_lastMessageID = '';
    //! The message Date to be used in the Date header.
    /*!
     * If empty, the current date will be added.
     * @var string
     */
    protected $_messageDate = '';

    //! Whether to keep SMTP connection open after each message.
    /*!
     * If this is set to true then to close the connection
     * requires an explicit call to smtpClose().
     * @var bool
     */
    protected $_SMTPKeepAlive = FALSE;
   
    //! The array of 'to' names and addresses.
   
    /*!
     * @var array
     */
    protected $_to = array();

    //! The array of 'cc' names and addresses.
    /*!
     * @var array
     */
    protected $_cc = array();

    //! The array of 'bcc' names and addresses.
    /*!
     * @var array
     */
    protected $_bcc = array();

    //! The array of reply-to names and addresses.
    /*!
     * @var array
     */
    protected $_replyTo = array();

    //! An array of all kinds of addresses.
    /*!
     * Includes all of $to, $cc, $bcc
     * @var array
     * @see PHPMailer::$to @see PHPMailer::$cc @see PHPMailer::$bcc
     */
    protected $_allRecipients = array();

    //! The array of attachments.
    /*!
     * @var array
     */
    protected $_attachments = array();

    //! The array of custom headers.
    /*!
     * @var array
     */
    protected $_customHeader = array();

    //! Unique ID used for message ID and boundaries.
    /*!
     * @var string
     */
    protected $_uniqueid = '';

    //! confirmation email address
    /*!
     * @var string
     */
    protected $_confirmReadingTo = '';

    //! pretend send mail
    /*!
     * @var bool
     */
    protected $_pretend = FALSE;

    //! @string with the CID type
    protected $_cidType = NULL ;

    //! @string with the CID value
    protected $_cid = NULL;

    //! @string auth method LOGIN, CRAM-MD5, PLAIN 
    protected $_authMethod = 'LOGIN';

    //! @string header 'from' address to set
    protected $_headerFromAddress = '';

    //! @string header 'from' friendly name part
    protected $_headerFromName = '';

    //! Constructor
    /*!
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function __construct() {
        parent::__construct();
    }

    //! destructor
    /*!
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function __destruct() {
        parent::__destruct();
    }

    //! Sets message type to HTML or plain.
    /*!
     * @param bool $isHtml True for HTML mode.
     */
    public function isHTML($isHtml = TRUE) {
        if ($isHtml) {
            $this->_contentType = 'text/html';
        } else {
            $this->_contentType = 'text/plain';
        }
    }

    //! Set the Sender $email address and friendly name
    /*!
     * @param string $username The username that will be used for the SMTP authentication 
     *
     * @param string $friendlyName The value that will be set as sender friendly name
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setSender($address, $friendlyName = '') {
        if (!isset($address) || !WizardValidator::isEmail($address)) {
            return FALSE;
        }
        $this->_sender = $address;
        $this->_senderName =  $friendlyName;

        return TRUE;
    }

    //! Set the 'header' from value - if it needs to be different than the Sender
    public function setHeaderFrom($address, $friendlyName = '') {
         if (!isset($address) || !WizardValidator::isEmail($address)) {
            return FALSE;
        }
        $this->_headerFromAddress = $address;
        $this->_headerFromName =  $friendlyName;

        return TRUE;
   
    }

    //! Set the "Username" for the SMTP authetication
    /*!
     * @param string $username The username that will be used for the SMTP authentication 
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setUsername($username) {
        if (!isset($username) || !WizardValidator::isEmail($username)) {
            return FALSE;
        }
        $this->_username = $username;
        return TRUE;
    }

    //! Set the "Password" for the SMTP authetication
    /*!
     * @param string $password The password that will be used for the SMTP authentication 
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setPassword($password) {
        if (!isset($password) || !WizardValidator::isStringNotEmpty($password)) {
            return FALSE;
        }
        $this->_password = $password;
        return TRUE;
    }

    //! Set the "Body" for the email could be HTML 
    /*!
     * @param string $body The email body that you want to add to your email
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setBody($body) {
        // this could be empty
        if (!isset($body) || !WizardValidator::isString($body)) {
            return FALSE;
        }
        $this->_body = $body;
        return TRUE;
    }

    //! Set the "Alternative Body" for the email in case the client cannot render HTML
    /*!
     * @param string $body The email Alternative body that you want to add to your email
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setAltBody($body) {
        if (!isset($body) || !WizardValidator::isStringNotEmpty($body)) {
            return FALSE;
        }
        $this->_altBody = $body;
        return TRUE;
    }

    //! Set the "Subject" for the email 
    /*!
     * @param string $subject The email Subject 
     * 
     * @return bool whatever we set the subject or not 
     */
    public function setSubject($subject) {
        // this could be empty
        if (!isset($subject) || !WizardValidator::isString($subject)) {
            return FALSE;
        }
        $this->_subject = $subject;
        return TRUE;
    }


    //! Add a "To" address.
    /*!
     * @param string $address The email address to send to
     * @param string $name
     *
     * @return bool true on success, false if address already used or invalid in some way
     */
    public function addTO($address, $name = '') {
        return $this->addOrEnqueueAnAddress('_to', $address, $name);
    }

    //! Add a "CC" address.
    /*!
     * @param string $address The email address to send to
     * @param string $name
     *
     * @return bool true on success, false if address already used or invalid in some way
     *
     * @note: This function works with the SMTP mailer on win32, not with the "mail" mailer.
     */
    public function addCC($address, $name = '') {
        return $this->addOrEnqueueAnAddress('_cc', $address, $name);
    }

    //! Add a "BCC" address.
    /*!
     * @param string $address The email address to send to
     * @param string $name
     *
     * @return bool true on success, false if address already used or invalid in some way
     *
     * @note: This function works with the SMTP mailer on win32, not with the "mail" mailer.
     */
    public function addBCC($address, $name = '') {
        return $this->addOrEnqueueAnAddress('_bcc', $address, $name);
    }

    //! Add a "Reply-To" address.
    /*!
     * @param string $address The email address to reply to
     * @param string $name
     *
     * @return bool true on success, false if address already used or invalid in some way
     */
    public function addReplyTo($address, $name = '') {
        return $this->addOrEnqueueAnAddress('_reply-To', $address, $name);
    }

    //! Add an address to one of the recipient arrays or to the ReplyTo array. Because PHPMailer
    /*!
     * @param string $kind One of 'to', 'cc', 'bcc', or 'ReplyTo'
     * @param string $address The email address to send, resp. to reply to
     * @param string $name
     *
     * @throws phpmailerException
     *
     * @return bool true on success, false if address already used or invalid in some way
     *
     * @note: can't validate addresses with an IDN without knowing the PHPMailer::$CharSet (that can still
     *        be modified after calling this function), addition of such addresses is delayed until send().
     *        Addresses that have been added already return false, but do not throw exceptions.
     */
    protected function addOrEnqueueAnAddress($kind, $address, $name) {
        $address = trim($address);
        $name = trim(preg_replace('/[\r\n]+/', '', $name)); //Strip breaks and trim
        if (filter_var($address, FILTER_VALIDATE_EMAIL) === FALSE) {
            // invalid email address .
            $error_message = 'Invalid_address' . " (addAnAddress $kind): $address";
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        $params = array($kind, $address, $name);
        // Immediately add standard addresses without IDN.
        return call_user_func_array(array($this, 'addAnAddress'), $params);
    }

    //! Add an address to one of the recipient arrays or to the ReplyTo array.
    /*!
     * @param string $kind One of 'to', 'cc', 'bcc', or 'ReplyTo'
     * @param string $address The email address to send, resp. to reply to
     * @param string $name
     *
     * @throws phpmailerException
     *
     * @return bool true on success, false if address already used or invalid in some way
     *
     * @note: Addresses that have been added already return false, but do not throw exceptions.
     */
    protected function addAnAddress($kind, $address, $name = '') {
        if (!in_array($kind, array('_to', '_cc', '_bcc', '_reply-To'))) {
            $error_message = 'Invalid recipient kind: ' . $kind;
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }
        if (filter_var($address, FILTER_VALIDATE_EMAIL) === FALSE) {
            $error_message = 'invalid_address' . " (addAnAddress $kind): $address";
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        if ($kind != '_reply-To') {
            if (!array_key_exists(strtolower($address), $this->_allRecipients)) {
                array_push($this->$kind, array($address, $name));
                $this->_allRecipients[strtolower($address)] = TRUE;
            } 
            return TRUE;
        } else {
            if (!array_key_exists(strtolower($address), $this->_replyTo)) {
                $this->_replyTo[strtolower($address)] = array($address, $name);
                return TRUE;
            }
        }
        return FALSE;
    }

    //! Send email
    /*!
     * @return bool TRUE if success, FALSE otherwise
     */
    public function send() {
        //sanity check
        if (!isset($this->_sender)) {
            $this->setError('Invalid member variable');
            return FALSE;
        }
        if ($this->preSend() === FALSE) {
            $error_message = 'Unable to prepare the message';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        // Attempt to connect
        if ($this->connect() !== TRUE) {
            $error_message = "There was a problem connecting";
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        // Send the EHLO/HELO 
        if (!$this->hello()) {
            $error_message = "There was a problem sending the EHLO/HELO";
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        // If the server is advertising STARTTLS capabilities and we want the message 
        // to be send over tls and PHP support stream_socket_client
        if ($this->getServerExt('STARTTLS') && $this->_tls && $this->_tls_support) {
            if (!$this->startTLS()) {
                $error_message = "There was a problem starting the TLS connection";
                $this->setError($error_message);
                $this->logMessage($error_message);
                return FALSE;
            }

            // Now that we started a TLS session we need to resend the EHELO
            if (!$this->hello()) {
                $error_message = "There was a problem sending the EHLO/HELO";
                $this->setError($error_message);
                $this->logMessage($error_message);
                return FALSE;
            }
        } 

        // If the server advertise CID and the CID Type and CID was set send the command 
        if ($this->getServerExt('CLIENTID')) {
            if (isset($this->_cid_type) && isset($this->_cid)) {
                if (!$this->sendCID($this->_cid_type, $this->_cid)) {
                    // XXX Maybe could use a better phrasing
                    $this->setError("The CLIENTID command was not accepted by the Mail Server");
                    return FALSE;
                }
            }
        }

        // If the username and password are set then try to authenticate 
        if (WizardValidator::isStringNotEmpty($this->_username) && WizardValidator::isStringNotEmpty($this->_password)) {
            if (!$this->authenticate($this->_username, $this->_password, $this->_authMethod)) {
                $error_message = "There was a problem authenticating. Server Replied: ";
                $error_message .= $this->_error['detail'];
                $this->setError($error_message);
                $this->logMessage($error_message);
                return FALSE;
            }
        }

        $this->mailFrom($this->_sender);
        
        // Attempt to send to all recipients
        foreach (array($this->_to, $this->_cc, $this->_bcc) as $togroup) {
            foreach ($togroup as $to) {
                if (!$this->recipient($to[0])) {
                    return FALSE;
                }
            }
        }
 
        // Send the actually data
        if (!$this->data($this->_MIMEHeader . $this->_MIMEBody)) {
            $error_message = 'Unable to build the message DATA';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        if ($this->_SMTPKeepAlive) {
            $this->reset();
        } else {
            $this->quit();
            $this->close();
        }
        return TRUE;
    }

    //! Prepare a message for sending.
    /*!
     * @throws phpmailerException
     *
     * @return bool
     */
    public function preSend() {
        if ((count($this->_to) + count($this->_cc) + count($this->_bcc)) < 1) {
            $error_message = 'There are no recipients';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        // Set whether the message is multipart/alternative
        if ($this->alternativeExists()) {
            $this->_contentType = 'multipart/alternative';
        }

        $this->setMessageType();

        // Create body before headers in case body makes changes to headers (e.g. altering transfer encoding)
        $this->_MIMEHeader = '';
        $this->_MIMEBody = $this->createBody();
        // createBody may have added some headers, so retain them
        $tempheaders = $this->_MIMEHeader;
        $this->_MIMEHeader = $this->createHeader();
        $this->_MIMEHeader .= $tempheaders;

        return TRUE;
    }
    
    //! Use this method to get the message header and body, this could be usefull if you want to save the message to draft or Send
    /*!
     * @return string
     */
    public function getData() {
        return $this->_MIMEHeader . $this->_MIMEBody;
    }

    //! Brief: Helper method that will allow us to inject a message into the local queue 
    /*!
     * @param strint $callerName the name of the App/Program that uses the qmail inject
     *
     * @returns bool whether the operation was successful
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function qmailInject($callerName = '') {
        if (!file_exists(self::QMAIL_INJECT_BIN)) {
            $error_message = 'Unable to find qmail inject bin [' . self::QMAIL_INJECT_BIN . ']';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return $error_messane;
        }
        if (isset($callerName) && is_string($callerName)) {
            $this->_qmail_inject_caller = $callerName;
        }
    
        // If this fails there is already a error message
        if (!$this->preSend()) {
            return $error_messane;
        }
        $emailData = $this->getData();
        if (empty($emailData)) {
            $error_message = 'There are no recipients';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return $error_messane;
        }
        $senderFlag = '';
        // If there is a sender let's make sure that we set it here
        if (!empty($this->_sender)) {
            $senderFlag = ' -f ' . escapeshellarg($this->_sender);
        }

        $handle = popen(self::QMAIL_INJECT_BIN . $senderFlag . ' 2>&1' , 'w');
        if (empty($handle)) {
            $error_message = 'Unable to open a pipe open [' . self::QMAIL_INJECT_BIN . ']';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return $error_messane;
        }
        // write the message to the QMAIL inject 
        if (!fwrite($handle, $emailData)) {
            pclose($handle);
            $error_message = 'Unable to write to the pipe [' . self::QMAIL_INJECT_BIN . ']';
            $this->setError($error_message);
            $this->logMessage($error_message);
            return $error_messane;
        }
        
        pclose($handle);
        return TRUE;
    }

    //! Create recipient headers.
    /*!
     * @param string $type
     * @param array $addr An array of recipient,
     * where each recipient is a 2-element indexed array with element 0 containing an address
     * and element 1 containing a name, like:
     * array(array('joe@example.com', 'Joe User'), array('zoe@example.com', 'Zoe User'))
     *
     * @return string
     */
    protected function addrAppend($type, $addr) {
        $addresses = array();
        if (isset($addr) && is_array($addr)) {
            foreach ($addr as $address) {
                $addresses[] = $this->addrFormat($address);
            }
            return $type . ': ' . implode(', ', $addresses) . $this->LE;
        }
    }

    //! Format an address for use in a message header.
    /*!
     * @param array $addr A 2-element indexed array, element 0 containing an address, element 1 containing a name
     *      like array('joe@example.com', 'Joe User')
     * @return string
     */
    public function addrFormat($addr) {
        if (empty($addr[1])) { // No name provided
            return $this->secureHeader($addr[0]);
        } else {
            return $this->encodeHeader($this->secureHeader($addr[1]), 'phrase') . ' <' . $this->secureHeader(
                $addr[0]
            ) . '>';
        }
    }

    //! Word-wrap message.
    /*!
     * @param string $message The message to wrap
     * @param integer $length The line length to wrap to
     * @param bool $qp_mode Whether to run in Quoted-Printable mode
     *
     * @return string
     *
     * @note: For use with mailers that do not automatically perform wrapping
     *        and for quoted-printable encoded messages.
     *        Original written by philippe.
     */
    public function wrapText($message, $length, $qp_mode = FALSE) {
        if ($qp_mode) {
            $soft_break = sprintf(' =%s', $this->LE);
        } else {
            $soft_break = $this->LE;
        }
        // If utf-8 encoding is used, we will need to make sure we don't
        // split multibyte characters when we wrap
        $is_utf8 = (strtolower($this->_charSet) == 'utf-8');
        $lelen = strlen($this->LE);
        $crlflen = strlen(self::CRLF);

        $message = $this->fixEOL($message);
        //Remove a trailing line break
        if (substr($message, -$lelen) == $this->LE) {
            $message = substr($message, 0, -$lelen);
        }

        //Split message into lines
        $lines = explode($this->LE, $message);
        //Message will be rebuilt in here
        $message = '';
        foreach ($lines as $line) {
            $words = explode(' ', $line);
            $buf = '';
            $firstword = TRUE;
            foreach ($words as $word) {
                if ($qp_mode and (strlen($word) > $length)) {
                    $space_left = $length - strlen($buf) - $crlflen;
                    if (!$firstword) {
                        if ($space_left > 20) {
                            $len = $space_left;
                            if ($is_utf8) {
                                $len = $this->utf8CharBoundary($word, $len);
                            } elseif (substr($word, $len - 1, 1) == '=') {
                                $len--;
                            } elseif (substr($word, $len - 2, 1) == '=') {
                                $len -= 2;
                            }
                            $part = substr($word, 0, $len);
                            $word = substr($word, $len);
                            $buf .= ' ' . $part;
                            $message .= $buf . sprintf('=%s', self::CRLF);
                        } else {
                            $message .= $buf . $soft_break;
                        }
                        $buf = '';
                    }
                    while (strlen($word) > 0) {
                        if ($length <= 0) {
                            break;
                        }
                        $len = $length;
                        if ($is_utf8) {
                            $len = $this->utf8CharBoundary($word, $len);
                        } elseif (substr($word, $len - 1, 1) == '=') {
                            $len--;
                        } elseif (substr($word, $len - 2, 1) == '=') {
                            $len -= 2;
                        }
                        $part = substr($word, 0, $len);
                        $word = substr($word, $len);

                        if (strlen($word) > 0) {
                            $message .= $part . sprintf('=%s', self::CRLF);
                        } else {
                            $buf = $part;
                        }
                    }
                } else {
                    $buf_o = $buf;
                    if (!$firstword) {
                        $buf .= ' ';
                    }
                    $buf .= $word;

                    if (strlen($buf) > $length and $buf_o != '') {
                        $message .= $buf_o . $soft_break;
                        $buf = $word;
                    }
                }
                $firstword = FALSE;
            }
            $message .= $buf . self::CRLF;
        }

        return $message;
    }

    //! Find the last character boundary prior to $maxLength in a utf-8
    /*!
     * quoted-printable encoded string.
     * @author: Original written by Colin Brown.
     *
     * @param string $encodedText utf-8 QP text
     * @param integer $maxLength Find the last character boundary prior to this length
     *
     * @return integer
     */
    public function utf8CharBoundary($encodedText, $maxLength) {
        $foundSplitPos = FALSE;
        $lookBack = 3;
        while (!$foundSplitPos) {
            $lastChunk = substr($encodedText, $maxLength - $lookBack, $lookBack);
            $encodedCharPos = strpos($lastChunk, '=');
            if (FALSE !== $encodedCharPos) {
                // Found start of encoded character byte within $lookBack block.
                // Check the encoded byte value (the 2 chars after the '=')
                $hex = substr($encodedText, $maxLength - $lookBack + $encodedCharPos + 1, 2);
                $dec = hexdec($hex);
                if ($dec < 128) {
                    // Single byte character.
                    // If the encoded char was found at pos 0, it will fit
                    // otherwise reduce maxLength to start of the encoded char
                    if ($encodedCharPos > 0) {
                        $maxLength = $maxLength - ($lookBack - $encodedCharPos);
                    }
                    $foundSplitPos = TRUE;
                } elseif ($dec >= 192) {
                    // First byte of a multi byte character
                    // Reduce maxLength to split at start of character
                    $maxLength = $maxLength - ($lookBack - $encodedCharPos);
                    $foundSplitPos = TRUE;
                } elseif ($dec < 192) {
                    // Middle byte of a multi byte character, look further back
                    $lookBack += 3;
                }
            } else {
                // No encoded character found
                $foundSplitPos = TRUE;
            }
        }
        return $maxLength;
    }

    //! Apply word wrapping to the message body.
    /*!
     * @note: Wraps the message body to the number of chars set in the WordWrap property.
     *        You should only do this to plain-text bodies as wrapping HTML tags may break them.
     *        This is called automatically by createBody(), so you don't need to call it yourself.
     */
    public function setWordWrap() {
        if ($this->_wordWrap < 1) {
            return;
        }

        switch ($this->message_type) {
            case 'alt':
            case 'alt_inline':
            case 'alt_attach':
            case 'alt_inline_attach':
                $this->_altBody = $this->wrapText($this->_altBody, $this->_wordWrap);
                break;
            default:
                $this->_body = $this->wrapText($this->_body, $this->_wordWrap);
                break;
        }
    }

    //! Assemble message headers.
    /*!
     * @return string The assembled headers
     */
    public function createHeader() {
        $result = '';

        if (!empty($this->_qmail_inject_caller)) {

        $receivedHeader = 'from ';
            $receivedHeader .= 'localhost (127.0.0.1)';
            $receivedHeader .= " by "  .  gethostbyaddr('127.0.0.1') . ' via ' . $this->_qmail_inject_caller;
            $result .= $this->headerLine('Received', $receivedHeader);
        }

        if ($this->_messageDate == '') {
            $this->_messageDate = self::rfcDate();
        }
        $result .= $this->headerLine('Date', $this->_messageDate);

        if (count($this->_to) > 0) {
            $result .= $this->addrAppend('To', $this->_to);
        } elseif (count($this->_cc) == 0) {
            $result .= $this->headerLine('To', 'undisclosed-recipients:;');
        }

        if (!empty($this->_headerFromAddress)) {
            $result .= $this->addrAppend('From', array(array($this->_headerFromAddress, $this->_headerFromName)));
        } else {
            $result .= $this->addrAppend('From', array(array($this->_sender, $this->_senderName)));
        }

        // sendmail and mail() extract Cc from the header before sending
        if (count($this->_cc) > 0) {
            $result .= $this->addrAppend('Cc', $this->_cc);
        }

        if (count($this->_replyTo) > 0) {
            $result .= $this->addrAppend('Reply-To', $this->_replyTo);
        }

        // mail() sets the subject itself
        $result .= $this->headerLine('Subject', $this->encodeHeader($this->secureHeader($this->_subject)));       

        // Only allow a custom message ID if it conforms to RFC 5322 section 3.6.4
        // https://tools.ietf.org/html/rfc5322#section-3.6.4
        if ('' != $this->_messageID and preg_match('/^<.*@.*>$/', $this->_messageID)) {
            $this->_lastMessageID = $this->_messageID;
        } else {
            // NOTE: this call is Ubuntu specific and requires that /etc/hosts be configured to our
            //       specifications.
            $host = gethostbyaddr('127.0.1.1');
            $this->_lastMessageID = sprintf('<%s@%s>', $this->_uniqueid, $host);
        }

        $result .= $this->headerLine('Message-ID', $this->_lastMessageID);

        if (!is_null($this->_priority)) {
            $result .= $this->headerLine('X-Priority', $this->_priority);
        }
       
        $result .= $this->headerLine('X-Mailer', 'Wizard PHP Mail Library 1.1');
    
        if ($this->_confirmReadingTo != '') {
            $result .= $this->headerLine('Disposition-Notification-To', '<' . $this->_confirmReadingTo . '>');
        }

        // Add custom headers
        foreach ($this->_customHeader as $header) {
            $result .= $this->headerLine(
                trim($header[0]),
                $this->encodeHeader(trim($header[1]))
            );
        }

        $result .= $this->headerLine('MIME-Version', '1.0');
        $result .= $this->getMailMIME();

        return $result;
    }

    //! Get the message MIME type headers.
    /*!
     * @return string
     */
    public function getMailMIME() {
        $result = '';
        $ismultipart = TRUE;
        switch ($this->message_type) {
            case 'inline':
                $result .= $this->headerLine('Content-Type', 'multipart/related;');
                $result .= $this->textLine("\tboundary=\"" . $this->boundary[1] . '"');
                break;
            case 'attach':
            case 'inline_attach':
            case 'alt_attach':
            case 'alt_inline_attach':
                $result .= $this->headerLine('Content-Type', 'multipart/mixed;');
                $result .= $this->textLine("\tboundary=\"" . $this->boundary[1] . '"');
                break;
            case 'alt':
            case 'alt_inline':
                $result .= $this->headerLine('Content-Type', 'multipart/alternative;');
                $result .= $this->textLine("\tboundary=\"" . $this->boundary[1] . '"');
                break;
            default:
                // Catches case 'plain': and case '':
                $result .= $this->textLine('Content-Type: ' . $this->_contentType . '; charset=' . $this->_charSet);
                $ismultipart = FALSE;
                break;
        }

        // RFC1341 part 5 says 7bit is assumed if not specified
        if ($this->_encoding != '7bit') {
            // RFC 2045 section 6.4 says multipart MIME parts may only use 7bit, 8bit or binary CTE
            if ($ismultipart) {
                if ($this->_encoding == '8bit') {
                    $result .= $this->headerLine('Content-Transfer-Encoding', '8bit');
                }
                // The only remaining alternatives are quoted-printable and base64, which are both 7bit compatible
            } else {
                $result .= $this->headerLine('Content-Transfer-Encoding', $this->_encoding);
            }
        }

        $result .= $this->LE;

        return $result;
    }

    //! Returns the whole MIME message.
    /*!
     * @see PHPMailer::preSend()
     *
     * @return string
     *
     * @note: Includes complete headers and body.
     *        Only valid post preSend().
     */
    public function getSentMIMEMessage() {
        return rtrim($this->_MIMEHeader . $this->_mailHeader, "\n\r") . self::CRLF . self::CRLF . $this->_MIMEBody;
    }

    //! Create unique ID
    /*!
     * @return string the unique id
     */
    protected function generateId() {
        return md5(uniqid(time()));
    }

    //! Assemble the message body.
    /*!
     * @throws phpmailerException
     *
     * @return string The assembled message body, empty string on failure
     */
    public function createBody() {
        $body = '';
        //Create unique IDs and preset boundaries
        $this->_uniqueid = $this->generateId();
        $this->boundary[1] = 'b1_' . $this->_uniqueid;
        $this->boundary[2] = 'b2_' . $this->_uniqueid;
        $this->boundary[3] = 'b3_' . $this->_uniqueid;

        $this->setWordWrap();

        $bodyEncoding = $this->_encoding;
        $bodyCharSet = $this->_charSet;
        //Can we do a 7-bit downgrade?
        if ($bodyEncoding == '8bit' and !$this->has8bitChars($this->_body)) {
            $bodyEncoding = '7bit';
            //All ISO 8859, Windows codepage and UTF-8 charsets are ascii compatible up to 7-bit
            $bodyCharSet = 'us-ascii';
        }
        //If lines are too long, and we're not already using an encoding that will shorten them,
        //change to quoted-printable transfer encoding for the body part only
        if ('base64' != $this->_encoding and self::hasLineLongerThanMax($this->_body)) {
            $bodyEncoding = 'quoted-printable';
        }

        $altBodyEncoding = $this->_encoding;
        $altBodyCharSet = $this->_charSet;
        //Can we do a 7-bit downgrade?
        if ($altBodyEncoding == '8bit' and !$this->has8bitChars($this->_altBody)) {
            $altBodyEncoding = '7bit';
            //All ISO 8859, Windows codepage and UTF-8 charsets are ascii compatible up to 7-bit
            $altBodyCharSet = 'us-ascii';
        }
        //If lines are too long, and we're not already using an encoding that will shorten them,
        //change to quoted-printable transfer encoding for the alt body part only
        if ('base64' != $altBodyEncoding and self::hasLineLongerThanMax($this->_altBody)) {
            $altBodyEncoding = 'quoted-printable';
        }
        //Use this as a preamble in all multipart message types
        $mimepre = "This is a multi-part message in MIME format." . $this->LE . $this->LE;
        switch ($this->message_type) {
            case 'inline':
                $body .= $mimepre;
                $body .= $this->getBoundary($this->boundary[1], $bodyCharSet, '', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->attachAll('inline', $this->boundary[1]);
                break;
            case 'attach':
                $body .= $mimepre;
                $body .= $this->getBoundary($this->boundary[1], $bodyCharSet, '', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->attachAll('attachment', $this->boundary[1]);
                break;
            case 'inline_attach':
                $body .= $mimepre;
                $body .= $this->textLine('--' . $this->boundary[1]);
                $body .= $this->headerLine('Content-Type', 'multipart/related;');
                $body .= $this->textLine("\tboundary=\"" . $this->boundary[2] . '"');
                $body .= $this->LE;
                $body .= $this->getBoundary($this->boundary[2], $bodyCharSet, '', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->attachAll('inline', $this->boundary[2]);
                $body .= $this->LE;
                $body .= $this->attachAll('attachment', $this->boundary[1]);
                break;
            case 'alt':
                $body .= $mimepre;
                $body .= $this->getBoundary($this->boundary[1], $altBodyCharSet, 'text/plain', $altBodyEncoding);
                $body .= $this->encodeString($this->_altBody, $altBodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->getBoundary($this->boundary[1], $bodyCharSet, 'text/html', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                if (!empty($this->Ical)) {
                    $body .= $this->getBoundary($this->boundary[1], '', 'text/calendar; method=REQUEST', '');
                    $body .= $this->encodeString($this->Ical, $this->_encoding);
                    $body .= $this->LE . $this->LE;
                }
                $body .= $this->endBoundary($this->boundary[1]);
                break;
            case 'alt_inline':
                $body .= $mimepre;
                $body .= $this->getBoundary($this->boundary[1], $altBodyCharSet, 'text/plain', $altBodyEncoding);
                $body .= $this->encodeString($this->_altBody, $altBodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->textLine('--' . $this->boundary[1]);
                $body .= $this->headerLine('Content-Type', 'multipart/related;');
                $body .= $this->textLine("\tboundary=\"" . $this->boundary[2] . '"');
                $body .= $this->LE;
                $body .= $this->getBoundary($this->boundary[2], $bodyCharSet, 'text/html', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->attachAll('inline', $this->boundary[2]);
                $body .= $this->LE;
                $body .= $this->endBoundary($this->boundary[1]);
                break;
            case 'alt_attach':
                $body .= $mimepre;
                $body .= $this->textLine('--' . $this->boundary[1]);
                $body .= $this->headerLine('Content-Type', 'multipart/alternative;');
                $body .= $this->textLine("\tboundary=\"" . $this->boundary[2] . '"');
                $body .= $this->LE;
                $body .= $this->getBoundary($this->boundary[2], $altBodyCharSet, 'text/plain', $altBodyEncoding);
                $body .= $this->encodeString($this->_altBody, $altBodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->getBoundary($this->boundary[2], $bodyCharSet, 'text/html', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->endBoundary($this->boundary[2]);
                $body .= $this->LE;
                $body .= $this->attachAll('attachment', $this->boundary[1]);
                break;
            case 'alt_inline_attach':
                $body .= $mimepre;
                $body .= $this->textLine('--' . $this->boundary[1]);
                $body .= $this->headerLine('Content-Type', 'multipart/alternative;');
                $body .= $this->textLine("\tboundary=\"" . $this->boundary[2] . '"');
                $body .= $this->LE;
                $body .= $this->getBoundary($this->boundary[2], $altBodyCharSet, 'text/plain', $altBodyEncoding);
                $body .= $this->encodeString($this->_altBody, $altBodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->textLine('--' . $this->boundary[2]);
                $body .= $this->headerLine('Content-Type', 'multipart/related;');
                $body .= $this->textLine("\tboundary=\"" . $this->boundary[3] . '"');
                $body .= $this->LE;
                $body .= $this->getBoundary($this->boundary[3], $bodyCharSet, 'text/html', $bodyEncoding);
                $body .= $this->encodeString($this->_body, $bodyEncoding);
                $body .= $this->LE . $this->LE;
                $body .= $this->attachAll('inline', $this->boundary[3]);
                $body .= $this->LE;
                $body .= $this->endBoundary($this->boundary[2]);
                $body .= $this->LE;
                $body .= $this->attachAll('attachment', $this->boundary[1]);
                break;
            default:
                // Catch case 'plain' and case '', applies to simple `text/plain` and `text/html` body content types
                //Reset the `Encoding` property in case we changed it for line length reasons
                $this->_encoding = $bodyEncoding;
                $body .= $this->encodeString($this->_body, $this->_encoding);
                break;
        }

        return $body;
    }

    //! Return the start of a message boundary.
    /*!
     * @param string $boundary
     * @param string $charSet
     * @param string $contentType
     * @param string $encoding
     *
     * @return string
     */
    protected function getBoundary($boundary, $charSet, $contentType, $encoding) {
        $result = '';
        if ($charSet == '') {
            $charSet = $this->_charSet;
        }
        if ($contentType == '') {
            $contentType = $this->_contentType;
        }
        if ($encoding == '') {
            $encoding = $this->_encoding;
        }
        $result .= $this->textLine('--' . $boundary);
        $result .= sprintf('Content-Type: %s; charset=%s', $contentType, $charSet);
        $result .= $this->LE;
        // RFC1341 part 5 says 7bit is assumed if not specified
        if ($encoding != '7bit') {
            $result .= $this->headerLine('Content-Transfer-Encoding', $encoding);
        }
        $result .= $this->LE;

        return $result;
    }

    //! Return the end of a message boundary.
    /*!
     * @param string $boundary
     *
     * @return string
     */
    protected function endBoundary($boundary) {
        return $this->LE . '--' . $boundary . '--' . $this->LE;
    }

    //! Set the message type.
    /*!
     * @return void
     * @note: PHPMailer only supports some preset message types, not arbitrary MIME structures.
     */
    protected function setMessageType() {
        $type = array();
        if ($this->alternativeExists()) {
            $type[] = 'alt';
        }
        if ($this->inlineImageExists()) {
            $type[] = 'inline';
        }
        if ($this->attachmentExists()) {
            $type[] = 'attach';
        }
        $this->message_type = implode('_', $type);
        if ($this->message_type == '') {
            //The 'plain' message_type refers to the message having a single body element, not that it is plain-text
            $this->message_type = 'plain';
        }
    }

    //! Format a header line.
    /*!
     * @param string $name
     * @param string $value
     *
     * @return string
     */
    public function headerLine($name, $value) {
        return $name . ': ' . $value . $this->LE;
    }

    //! Return a formatted mail line.
    /*!
     * @param string $value
     *
     * @return string
     */
    public function textLine($value) {
        return $value . $this->LE;
    }

    //! Add an attachment from a path on the filesystem.
    /*!
     * @param string $path Path to the attachment.
     * @param string $name Overrides the attachment name.
     * @param string $encoding File encoding (see $Encoding).
     * @param string $type File extension (MIME) type.
     * @param string $disposition Disposition to use
     * @param bool $isData flag to determine if the path is a file path or the content of the file 
     *
     * @throws phpmailerException
     *
     * @return bool TRUE on success, FALSE otherwise
     */
    public function addAttachment($path, $name = '', $encoding = 'base64', $type = '', $disposition = 'attachment', $isData = FALSE) {
        if ($isData === FALSE && (!is_readable($path) || !is_file($path))) {
            $error_message = "Path [$path] is not readable or is not a file";
            $this->setError($error_message);
            $this->logMessage($error_message);
            return FALSE;
        }

        // If a MIME type is not specified, try to work it out from the file name
        if ($type == '') {
            $type = self::filenameToType($path);
        }
        // As stated in the definition of the Content-Transfer-Encoding field, no encoding other than "7bit", "8bit", 
        // or "binary" is permitted for messages or parts of type "message"
        if ($type === 'message/rfc822' && $encoding === 'base64') {
            $encoding = '8bit';
         }

        $filename = basename($path);
        if ($name == '') {
            $name = $filename;
        }



        $this->_attachments[] = array(
            0 => $path,
            1 => $filename,
            2 => $name,
            3 => $encoding,
            4 => $type,
            5 => $isData, // isStringAttachment 
            6 => $disposition,
            7 => 0
        );
       
        return TRUE;
    }

    //! Return the array of attachments.
    /*!
     * @return array the array of attachments
     */
    public function getAttachments() {
        return $this->_attachments;
    }

    //! Attach all file, string, and binary attachments to the message.
    /*!
     * @param string $disposition_type
     * @param string $boundary
     *
     * @return string empty string on failure
     */
    protected function attachAll($disposition_type, $boundary) {
        // Return text of body
        $mime = array();
        $cidUniq = array();
        $incl = array();

        // Add all attachments
        foreach ($this->_attachments as $attachment) {
            // Check if it is a valid disposition_filter
            if ($attachment[6] == $disposition_type) {
                // Check for string attachment
                $string = '';
                $path = '';
                $bString = $attachment[5];
                if ($bString) {
                    $string = $attachment[0];
                } else {
                    $path = $attachment[0];
                }

                $inclhash = md5(serialize($attachment));
                if (in_array($inclhash, $incl)) {
                    continue;
                }
                $incl[] = $inclhash;
                $name = $attachment[2];
                $encoding = $attachment[3];
                $type = $attachment[4];
                $disposition = $attachment[6];
                $cid = $attachment[7];
                if ($disposition == 'inline' && array_key_exists($cid, $cidUniq)) {
                    continue;
                }
                $cidUniq[$cid] = TRUE;

                $mime[] = sprintf('--%s%s', $boundary, $this->LE);
                //Only include a filename property if we have one
                if (!empty($name)) {
                    $mime[] = sprintf(
                        'Content-Type: %s; name="%s"%s',
                        $type,
                        $this->encodeHeader($this->secureHeader($name)),
                        $this->LE
                    );
                } else {
                    $mime[] = sprintf(
                        'Content-Type: %s%s',
                        $type,
                        $this->LE
                    );
                }
                // RFC1341 part 5 says 7bit is assumed if not specified
                if ($encoding != '7bit') {
                    $mime[] = sprintf('Content-Transfer-Encoding: %s%s', $encoding, $this->LE);
                }

                if ($disposition == 'inline') {
                    $mime[] = sprintf('Content-ID: <%s>%s', $cid, $this->LE);
                }

                // If a filename contains any of these chars, it should be quoted,
                // but not otherwise: RFC2183 & RFC2045 5.1
                // Fixes a warning in IETF's msglint MIME checker
                // Allow for bypassing the Content-Disposition header totally
                if (!(empty($disposition))) {
                    $encoded_name = $this->encodeHeader($this->secureHeader($name));
                    if (preg_match('/[ \(\)<>@,;:\\"\/\[\]\?=]/', $encoded_name)) {
                        $mime[] = sprintf(
                            'Content-Disposition: %s; filename="%s"%s',
                            $disposition,
                            $encoded_name,
                            $this->LE . $this->LE
                        );
                    } else {
                        if (!empty($encoded_name)) {
                            $mime[] = sprintf(
                                'Content-Disposition: %s; filename=%s%s',
                                $disposition,
                                $encoded_name,
                                $this->LE . $this->LE
                            );
                        } else {
                            $mime[] = sprintf(
                                'Content-Disposition: %s%s',
                                $disposition,
                                $this->LE . $this->LE
                            );
                        }
                    }
                } else {
                    $mime[] = $this->LE;
                }

                // Encode as string attachment
                if ($bString) {
                    $mime[] = $this->encodeString($string, $encoding);
                    $mime[] = $this->LE . $this->LE;
                } else {
                    $mime[] = $this->encodeFile($path, $encoding);
                    $mime[] = $this->LE . $this->LE;
                }
            }
        }

        $mime[] = sprintf('--%s--%s', $boundary, $this->LE);

        return implode('', $mime);
    }

    //! Encode a file attachment in requested format.
    /*!
     * @param string $path The full path to the file
     * @param string $encoding The encoding to use; one of 'base64', '7bit', '8bit', 'binary', 'quoted-printable'
     *
     * @throws phpmailerException
     *
     * @return string encoded file attachment, empty string on failure
     */
    protected function encodeFile($path, $encoding = 'base64') {
            if (!is_readable($path)) {

                $this->logMessage("Unable to open the file");
                return '';
                
            }
            $magic_quotes = get_magic_quotes_runtime();
            if ($magic_quotes) {
                if (version_compare(PHP_VERSION, '5.3.0', '<')) {
                    set_magic_quotes_runtime(FALSE);
                } else {
                    //Doesn't exist in PHP 5.4, but we don't need to check because
                    //get_magic_quotes_runtime always returns false in 5.4+
                    //so it will never get here
                    ini_set('magic_quotes_runtime', FALSE);
                }
            }
            $file_buffer = file_get_contents($path);
            $file_buffer = $this->encodeString($file_buffer, $encoding);
            if ($magic_quotes) {
                if (version_compare(PHP_VERSION, '5.3.0', '<')) {
                    set_magic_quotes_runtime($magic_quotes);
                } else {
                    ini_set('magic_quotes_runtime', $magic_quotes);
                }
            }
            return $file_buffer;
        
    }

    //! Encode a string in requested format.
    /*!
     * @param string $str The text to encode
     * @param string $encoding The encoding to use; one of 'base64', '7bit', '8bit', 'binary', 'quoted-printable'
     * @return string encoded string, empty string on failure
     */
    public function encodeString($str, $encoding = 'base64') {
        $encoded = '';
        switch (strtolower($encoding)) {
            case 'base64':
                $encoded = chunk_split(base64_encode($str), 76, $this->LE);
                break;
            case '7bit':
            case '8bit':
                $encoded = $this->fixEOL($str);
                // Make sure it ends with a line break
                if (substr($encoded, -(strlen($this->LE))) != $this->LE) {
                    $encoded .= $this->LE;
                }
                break;
            case 'binary':
                $encoded = $str;
                break;
            case 'quoted-printable':
                $encoded = $this->encodeQP($str);
                break;
            default:
                $this->setError('encoding ' . $encoding);
                break;
        }
        return $encoded;
    }

    //! Encode a header string optimally.
    /*!
     * @param string $str
     * @param string $position
     *
     * @return string encoded header
     *
     * @note: Picks shortest of Q, B, quoted-printable or none. 
     */
    public function encodeHeader($str, $position = 'text') {
        $matchcount = 0;
        switch (strtolower($position)) {
            case 'phrase':
                if (!preg_match('/[\200-\377]/', $str)) {
                    // Can't use addslashes as we don't know the value of magic_quotes_sybase
                    $encoded = addcslashes($str, "\0..\37\177\\\"");
                    if (($str == $encoded) && !preg_match('/[^A-Za-z0-9!#$%&\'*+\/=?^_`{|}~ -]/', $str)) {
                        return ($encoded);
                    } else {
                        return ("\"$encoded\"");
                    }
                }
                $matchcount = preg_match_all('/[^\040\041\043-\133\135-\176]/', $str, $matches);
                break;
            /** @noinspection PhpMissingBreakStatementInspection */
            case 'comment':
                $matchcount = preg_match_all('/[()"]/', $str, $matches);
                // Intentional fall-through
            case 'text':
            default:
                $matchcount += preg_match_all('/[\000-\010\013\014\016-\037\177-\377]/', $str, $matches);
                break;
        }

        //There are no chars that need encoding
        if ($matchcount == 0) {
            return ($str);
        }

        $maxlen = 75 - 7 - strlen($this->_charSet);
        // Try to select the encoding which should produce the shortest output
        if ($matchcount > strlen($str) / 3) {
            // More than a third of the content will need encoding, so B encoding will be most efficient
            $encoding = 'B';
            if (function_exists('mb_strlen') && $this->hasMultiBytes($str)) {
                // Use a custom function which correctly encodes and wraps long
                // multibyte strings without breaking lines within a character
                $encoded = $this->base64EncodeWrapMB($str, "\n");
            } else {
                $encoded = base64_encode($str);
                $maxlen -= $maxlen % 4;
                $encoded = trim(chunk_split($encoded, $maxlen, "\n"));
            }
        } else {
            $encoding = 'Q';
            $encoded = $this->encodeQ($str, $position);
            $encoded = $this->wrapText($encoded, $maxlen, TRUE);
            $encoded = str_replace('=' . self::CRLF, "\n", trim($encoded));
        }

        $encoded = preg_replace('/^(.*)$/m', ' =?' . $this->_charSet . "?$encoding?\\1?=", $encoded);
        $encoded = trim(str_replace("\n", $this->LE, $encoded));

        return $encoded;
    }

    //! Check if a string contains multi-byte characters.
    /*!
     * @param string $str multi-byte text to wrap encode
     *
     * @return bool whether it has multi-byte characeters
     */
    public function hasMultiBytes($str) {
        if (function_exists('mb_strlen')) {
            return (strlen($str) > mb_strlen($str, $this->_charSet));
        } else { // Assume no multibytes (we can't handle without mbstring functions anyway)
            return FALSE;
        }
    }

    //! Check if a string contains 8-bit characters
    /*!
     * @param string $text
     *
     * @return bool whether it has 8-bit characters
     */
    public function has8bitChars($text) {
        return (bool)preg_match('/[\x80-\xFF]/', $text);
    }

    //! Encode and wrap long multibyte strings for mail headers
    /*!
     * @link http://www.php.net/manual/en/function.mb-encode-mimeheader.php#60283
     *
     * @param string $str multi-byte text to wrap encode
     * @param string $linebreak string to use as linefeed/end-of-line
     *
     * @return string
     *
     * @note:  without breaking lines within a character.
     *         Adapted from a function by paravoid
     */
    public function base64EncodeWrapMB($str, $linebreak = NULL) {
        $start = '=?' . $this->_charSet . '?B?';
        $end = '?=';
        $encoded = '';
        if ($linebreak === NULL) {
            $linebreak = $this->LE;
        }

        $mb_length = mb_strlen($str, $this->_charSet);
        // Each line must have length <= 75, including $start and $end
        $length = 75 - strlen($start) - strlen($end);
        // Average multi-byte ratio
        $ratio = $mb_length / strlen($str);
        // Base64 has a 4:3 ratio
        $avgLength = floor($length * $ratio * .75);

        for ($i = 0; $i < $mb_length; $i += $offset) {
            $lookBack = 0;
            do {
                $offset = $avgLength - $lookBack;
                $chunk = mb_substr($str, $i, $offset, $this->_charSet);
                $chunk = base64_encode($chunk);
                $lookBack++;
            } while (strlen($chunk) > $length);
            $encoded .= $chunk . $linebreak;
        }

        // Chomp the last linefeed
        $encoded = substr($encoded, 0, -strlen($linebreak));
        return $encoded;
    }

    //! Encode a string in quoted-printable format.
    /*!
     *
     * @param string $string The text to encode
     * @param integer $line_max Number of chars allowed on a line before wrapping
     *
     * @return string
     *
     * @note: According to RFC2045 section 6.7.
     *
     * @link http://www.php.net/manual/en/function.quoted-printable-decode.php#89417 Adapted from this comment
     */
    public function encodeQP($string, $line_max = 76) {
        // Use native function if it's available (>= PHP5.3)
        if (function_exists('quoted_printable_encode')) {
            return quoted_printable_encode($string);
        }
        // Fall back to a pure PHP implementation
        $string = str_replace(
            array('%20', '%0D%0A.', '%0D%0A', '%'),
            array(' ', "\r\n=2E", "\r\n", '='),
            rawurlencode($string)
        );
        return preg_replace('/[^\r\n]{' . ($line_max - 3) . '}[^=\r\n]{2}/', "$0=\r\n", $string);
    }

    //! Encode a string using Q encoding.
    /*!
     * @param string $str the text to encode
     * @param string $position Where the text is going to be used, see the RFC for what that means
     *
     * @return string
     *
     * @link http://tools.ietf.org/html/rfc2047
     */
    public function encodeQ($str, $position = 'text') {
        // There should not be any EOL in the string
        $pattern = '';
        $encoded = str_replace(array("\r", "\n"), '', $str);
        switch (strtolower($position)) {
            case 'phrase':
                // RFC 2047 section 5.3
                $pattern = '^A-Za-z0-9!*+\/ -';
                break;
            /** @noinspection PhpMissingBreakStatementInspection */
            case 'comment':
                // RFC 2047 section 5.2
                $pattern = '\(\)"';
                // intentional fall-through
                // for this reason we build the $pattern without including delimiters and []
            case 'text':
            default:
                // RFC 2047 section 5.1
                // Replace every high ascii, control, =, ? and _ characters
                $pattern = '\000-\011\013\014\016-\037\075\077\137\177-\377' . $pattern;
                break;
        }
        $matches = array();
        if (preg_match_all("/[{$pattern}]/", $encoded, $matches)) {
            // If the string contains an '=', make sure it's the first thing we replace
            // so as to avoid double-encoding
            $eqkey = array_search('=', $matches[0]);
            if (FALSE !== $eqkey) {
                unset($matches[0][$eqkey]);
                array_unshift($matches[0], '=');
            }
            foreach (array_unique($matches[0]) as $char) {
                $encoded = str_replace($char, '=' . sprintf('%02X', ord($char)), $encoded);
            }
        }
        // Replace every spaces to _ (more readable than =20)
        return str_replace(' ', '_', $encoded);
    }

    //! Add a string or binary attachment (non-filesystem).
    /*!
     * @param string $string String attachment data.
     * @param string $filename Name of the attachment.
     * @param string $encoding File encoding (see $Encoding).
     * @param string $type File extension (MIME) type.
     * @param string $disposition Disposition to use
     *
     * @note: This method can be used to attach ascii or binary data, 
     *        such as a BLOB record from a database 
     */
    public function addStringAttachment(
        $string,
        $filename,
        $encoding = 'base64',
        $type = '',
        $disposition = 'attachment'
    ) {
        // If a MIME type is not specified, try to work it out from the file name
        if ($type == '') {
            $type = self::filenameToType($filename);
        }
        // Append to $attachment array
        $this->_attachments[] = array(
            0 => $string,
            1 => $filename,
            2 => basename($filename),
            3 => $encoding,
            4 => $type,
            5 => TRUE, // isStringAttachment
            6 => $disposition,
            7 => 0
        );
    }

    //! Add an embedded (inline) attachment from a file.
    /*!
     * @param string $path Path to the attachment.
     * @param string $cid Content ID of the attachment; Use this to reference
     *        the content when using an embedded image in HTML.
     * @param string $name Overrides the attachment name.
     * @param string $encoding File encoding (see $Encoding).
     * @param string $type File MIME type.
     * @param string $disposition Disposition to use
     *
     * @return bool True on successfully adding an attachment
     *
     * @note: This can include images, sounds, and just about any other document type.
     *        These differ from 'regular' attachments in that they are intended to be
     *        displayed inline with the message, not just attached for download.
     *        This is used in HTML messages that embed the images
     *        the HTML refers to using the $cid value.

     */
    public function addEmbeddedImage($path, $cid, $name = '', $encoding = 'base64', $type = '', $disposition = 'inline') {
        if (!@is_file($path)) {
            $this->setError('file_access: ' . $path);
            return FALSE;
        }

        // If a MIME type is not specified, try to work it out from the file name
        if ($type == '') {
            $type = self::filenameToType($path);
        }

        $filename = basename($path);
        if ($name == '') {
            $name = $filename;
        }

        // Append to $attachment array
        $this->_attachments[] = array(
            0 => $path,
            1 => $filename,
            2 => $name,
            3 => $encoding,
            4 => $type,
            5 => FALSE, // isStringAttachment
            6 => $disposition,
            7 => $cid
        );
        return TRUE;
    }

    //! Add an embedded stringified attachment.
    /*!
     * @param string $string The attachment binary data.
     * @param string $cid Content ID of the attachment; Use this to reference
     *        the content when using an embedded image in HTML.
     * @param string $name
     * @param string $encoding File encoding (see $Encoding).
     * @param string $type MIME type.
     * @param string $disposition Disposition to use
     *
     * @return bool True on successfully adding an attachment
     *
     * @note: This can include images, sounds, and just about any other document type.
     *        Be sure to set the $type to an image type for images:
     *        JPEG images use 'image/jpeg', GIF uses 'image/gif', PNG uses 'image/png'.
     */
    public function addStringEmbeddedImage(
        $string,
        $cid,
        $name = '',
        $encoding = 'base64',
        $type = '',
        $disposition = 'inline'
    ) {
        // If a MIME type is not specified, try to work it out from the name
        if ($type == '' and !empty($name)) {
            $type = self::filenameToType($name);
        }

        // Append to $attachment array
        $this->_attachments[] = array(
            0 => $string,
            1 => $name,
            2 => $name,
            3 => $encoding,
            4 => $type,
            5 => TRUE, // isStringAttachment
            6 => $disposition,
            7 => $cid
        );
        return TRUE;
    }

    //! Check if an inline attachment is present.
    /*!
     * @return bool whether inline image exists or not
     */
    public function inlineImageExists() {
        foreach ($this->_attachments as $attachment) {
            if ($attachment[6] == 'inline') {
                return TRUE;
            }
        }
        return FALSE;
    }

    //! Check if an attachment (non-inline) is present.
    /*!
     * @return bool whether non-inline attachment exists or not
     */
    public function attachmentExists() {
        foreach ($this->_attachments as $attachment) {
            if ($attachment[6] == 'attachment') {
                return TRUE;
            }
        }
        return FALSE;
    }

    //! Check if this message has an alternative body set.
    /*!
     * @return bool whether an alternative body is set or not
     */
    public function alternativeExists() {
        return !empty($this->_altBody);
    }

    //! Clear all recipient types.
    /*!
     */
    public function clearAllRecipients() {
        $this->_to = array();
        $this->_cc = array();
        $this->_bcc = array();
        $this->_allRecipients = array();
    }

    //! Clear all filesystem, string, and binary attachments.
    /*!
     */
    public function clearAttachments() {
        $this->_attachments = array();
    }

    //! Clear all custom headers.
    /*!
     */
    public function clearCustomHeaders() {
        $this->_customHeader = array();
    }

    //! Return an RFC 822 formatted date.
    /*!
     * @return string the RFC 822 formatted date
     */
    public static function rfcDate() {
        // Set the time zone to whatever the default is to avoid 500 errors
        // Will default to UTC if it's not set properly in php.ini
        date_default_timezone_set(@date_default_timezone_get());
        return date('D, j M Y H:i:s O');
    }

    //! Ensure consistent line endings in a string.
    /*!
     * @param string $str String to fixEOL
     *
     * @return string the resulting string
     *
     * @note: Changes every end of line from CRLF, CR or LF to $this->LE.
     */
    public function fixEOL($str) {
        // Normalise to \n
        $nstr = str_replace(array("\r\n", "\r"), "\n", $str);
        // Now convert LE as needed
        if ($this->LE !== "\n") {
            $nstr = str_replace("\n", $this->LE, $nstr);
        }
        return $nstr;
    }

    //! Add a custom header.
    /*!
     * @param string $name Custom header name
     * @param string $value Header value
     *
     * @note: $name value can be overloaded to contain
     *        both header name and value (name:value)
     */
    public function addCustomHeader($name, $value = NULL) {
        if ($value === NULL) {
            // Value passed in as name:value
            $this->_customHeader[] = explode(':', $name, 2);
        } else {
            $this->_customHeader[] = array($name, $value);
        }
    }

    //! Returns all custom headers.
    /*!
     * @return array the array of custom headers
     */
    public function getCustomHeaders() {
        return $this->_customHeader;
    }

    //! Create a message body from an HTML string.
    /*!
     * @param string $message HTML message string
     * @param string $basedir base directory for relative paths to images
     * @param bool|callable $advanced Whether to use the internal HTML to text converter
     *    or your own custom converter @see PHPMailer::html2text()
     *
     * @return string $message The transformed message Body
     *
     * @note: Automatically inlines images and creates a plain-text version by converting the HTML,
     *        overwriting any existing values in Body and AltBody.
     *        $basedir is used when handling relative image paths, e.g. <img src="images/a.png">
     *        will look for an image file in $basedir/images/a.png and convert it to inline.
     *        If you don't want to apply these transformations to your HTML, just set Body and AltBody yourself.
     */
    public function msgHTML($message, $basedir = '', $advanced = FALSE) {
        preg_match_all('/(src|background)=["\'](.*)["\']/Ui', $message, $images);
        if (array_key_exists(2, $images)) {
            foreach ($images[2] as $imgindex => $url) {
                // Convert data URIs into embedded images
                if (preg_match('#^data:(image[^;,]*)(;base64)?,#', $url, $match)) {
                    $data = substr($url, strpos($url, ','));
                    if ($match[2]) {
                        $data = base64_decode($data);
                    } else {
                        $data = rawurldecode($data);
                    }
                    $cid = md5($url) . '@wizard.0'; // RFC2392 S 2
                    if ($this->addStringEmbeddedImage($data, $cid, 'embed' . $imgindex, 'base64', $match[1])) {
                        $message = str_replace(
                            $images[0][$imgindex],
                            $images[1][$imgindex] . '="cid:' . $cid . '"',
                            $message
                        );
                    }
                } elseif (substr($url, 0, 4) !== 'cid:' && !preg_match('#^[a-z][a-z0-9+.-]*://#i', $url)) {
                    // Do not change urls for absolute images (thanks to corvuscorax)
                    // Do not change urls that are already inline images
                    $filename = basename($url);
                    $directory = dirname($url);
                    if ($directory == '.') {
                        $directory = '';
                    }
                    $cid = md5($url) . '@wizard.0'; // RFC2392 S 2
                    if (strlen($basedir) > 1 && substr($basedir, -1) != '/') {
                        $basedir .= '/';
                    }
                    if (strlen($directory) > 1 && substr($directory, -1) != '/') {
                        $directory .= '/';
                    }
                    if ($this->addEmbeddedImage(
                        $basedir . $directory . $filename,
                        $cid,
                        $filename,
                        'base64',
                        self::_mime_types((string)self::mb_pathinfo($filename, PATHINFO_EXTENSION))
                    )
                    ) {
                        $message = preg_replace(
                            '/' . $images[1][$imgindex] . '=["\']' . preg_quote($url, '/') . '["\']/Ui',
                            $images[1][$imgindex] . '="cid:' . $cid . '"',
                            $message
                        );
                    }
                }
            }
        }
        $this->isHTML(TRUE);
        // Convert all message body line breaks to CRLF, makes quoted-printable encoding work much better
        $this->_body = $this->normalizeBreaks($message);
        $this->_altBody = $this->normalizeBreaks($this->html2text($message, $advanced));
        if (!$this->alternativeExists()) {
            $this->_altBody = 'To view this email message, open it in a program that understands HTML!' .
                self::CRLF . self::CRLF;
        }
        return $this->_body;
    }

    //! Convert an HTML string into plain text.
    /*!
     * @param string $html The HTML text to convert
     * @param bool|callable $advanced Any bool value to use the internal converter,
     *   or provide your own callable for custom conversion.
     *
     * @return string
     *
     * @note: This is used by msgHTML().
     *        older versions of this function used a bundled advanced converter
     *        which was been removed for license reasons in #232.
     *        Example usage:
     *        <code>
     *        // Use default conversion
     *        $plain = $mail->html2text($html);
     *        // Use your own custom converter
     *        $plain = $mail->html2text($html, function($html) {
     *            $converter = new MyHtml2text($html);
     *            return $converter->get_text();
     *        });
     *        </code>
     */
    public function html2text($html, $advanced = FALSE) {
        if (is_callable($advanced)) {
            return call_user_func($advanced, $html);
        }

        $html = preg_replace('/<br *\/? *>/msi', "\n", $html);
        return html_entity_decode(
            trim(strip_tags(preg_replace('/<(head|title|style|script)[^>]*>.*?<\/\\1>/msi', '', $html))),
            ENT_QUOTES,
            $this->_charSet
        );
    }

    //! Get the MIME type for a file extension.
    /*!
     * @param string $ext File extension
     *
     * @return string MIME type of file.
     */
    public static function _mime_types($ext = '') {
        $mimes = array(
            'xl'    => 'application/excel',
            'js'    => 'application/javascript',
            'hqx'   => 'application/mac-binhex40',
            'cpt'   => 'application/mac-compactpro',
            'bin'   => 'application/macbinary',
            'doc'   => 'application/msword',
            'word'  => 'application/msword',
            'xlsx'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'xltx'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
            'potx'  => 'application/vnd.openxmlformats-officedocument.presentationml.template',
            'ppsx'  => 'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
            'pptx'  => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
            'sldx'  => 'application/vnd.openxmlformats-officedocument.presentationml.slide',
            'docx'  => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'dotx'  => 'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
            'xlam'  => 'application/vnd.ms-excel.addin.macroEnabled.12',
            'xlsb'  => 'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
            'class' => 'application/octet-stream',
            'dll'   => 'application/octet-stream',
            'dms'   => 'application/octet-stream',
            'exe'   => 'application/octet-stream',
            'lha'   => 'application/octet-stream',
            'lzh'   => 'application/octet-stream',
            'psd'   => 'application/octet-stream',
            'sea'   => 'application/octet-stream',
            'so'    => 'application/octet-stream',
            'oda'   => 'application/oda',
            'pdf'   => 'application/pdf',
            'ai'    => 'application/postscript',
            'eps'   => 'application/postscript',
            'ps'    => 'application/postscript',
            'smi'   => 'application/smil',
            'smil'  => 'application/smil',
            'mif'   => 'application/vnd.mif',
            'xls'   => 'application/vnd.ms-excel',
            'ppt'   => 'application/vnd.ms-powerpoint',
            'wbxml' => 'application/vnd.wap.wbxml',
            'wmlc'  => 'application/vnd.wap.wmlc',
            'dcr'   => 'application/x-director',
            'dir'   => 'application/x-director',
            'dxr'   => 'application/x-director',
            'dvi'   => 'application/x-dvi',
            'gtar'  => 'application/x-gtar',
            'php3'  => 'application/x-httpd-php',
            'php4'  => 'application/x-httpd-php',
            'php'   => 'application/x-httpd-php',
            'phtml' => 'application/x-httpd-php',
            'phps'  => 'application/x-httpd-php-source',
            'swf'   => 'application/x-shockwave-flash',
            'sit'   => 'application/x-stuffit',
            'tar'   => 'application/x-tar',
            'tgz'   => 'application/x-tar',
            'xht'   => 'application/xhtml+xml',
            'xhtml' => 'application/xhtml+xml',
            'zip'   => 'application/zip',
            'mid'   => 'audio/midi',
            'midi'  => 'audio/midi',
            'mp2'   => 'audio/mpeg',
            'mp3'   => 'audio/mpeg',
            'mpga'  => 'audio/mpeg',
            'aif'   => 'audio/x-aiff',
            'aifc'  => 'audio/x-aiff',
            'aiff'  => 'audio/x-aiff',
            'ram'   => 'audio/x-pn-realaudio',
            'rm'    => 'audio/x-pn-realaudio',
            'rpm'   => 'audio/x-pn-realaudio-plugin',
            'ra'    => 'audio/x-realaudio',
            'wav'   => 'audio/x-wav',
            'bmp'   => 'image/bmp',
            'gif'   => 'image/gif',
            'jpeg'  => 'image/jpeg',
            'jpe'   => 'image/jpeg',
            'jpg'   => 'image/jpeg',
            'png'   => 'image/png',
            'tiff'  => 'image/tiff',
            'tif'   => 'image/tiff',
            'eml'   => 'message/rfc822',
            'css'   => 'text/css',
            'html'  => 'text/html',
            'htm'   => 'text/html',
            'shtml' => 'text/html',
            'log'   => 'text/plain',
            'text'  => 'text/plain',
            'txt'   => 'text/plain',
            'rtx'   => 'text/richtext',
            'rtf'   => 'text/rtf',
            'vcf'   => 'text/vcard',
            'vcard' => 'text/vcard',
            'xml'   => 'text/xml',
            'xsl'   => 'text/xml',
            'mpeg'  => 'video/mpeg',
            'mpe'   => 'video/mpeg',
            'mpg'   => 'video/mpeg',
            'mov'   => 'video/quicktime',
            'qt'    => 'video/quicktime',
            'rv'    => 'video/vnd.rn-realvideo',
            'avi'   => 'video/x-msvideo',
            'movie' => 'video/x-sgi-movie'
        );
        if (array_key_exists(strtolower($ext), $mimes)) {
            return $mimes[strtolower($ext)];
        }
        return 'application/octet-stream';
    }

    //! Map a file name to a MIME type.
    /*!
     * @param string $filename A file name or full path, does not need to exist as a file
     *
     * @return string the MIME type
     *
     * @note: Defaults to 'application/octet-stream', i.e.. arbitrary binary data.
     */
    public static function filenameToType($filename) {
        // In case the path is a URL, strip any query string before getting extension
        $qpos = strpos($filename, '?');
        if (FALSE !== $qpos) {
            $filename = substr($filename, 0, $qpos);
        }
        $pathinfo = self::mb_pathinfo($filename);
        return self::_mime_types($pathinfo['extension']);
    }

    //! Multi-byte-safe pathinfo replacement.
    /*!
     * Drop-in replacement for pathinfo(), but multibyte-safe, cross-platform-safe, old-version-safe.
     * Works similarly to the one in PHP >= 5.2.0
     * @link http://www.php.net/manual/en/function.pathinfo.php#107461
     * @param string $path A filename or path, does not need to exist as a file
     * @param integer|string $options Either a PATHINFO_* constant,
     *      or a string name to return only the specified piece, allows 'filename' to work on PHP < 5.2
     * @return string|array
     * @static
     */
    public static function mb_pathinfo($path, $options = NULL) {
        $ret = array('dirname' => '', 'basename' => '', 'extension' => '', 'filename' => '');
        $pathinfo = array();
        if (preg_match('%^(.*?)[\\\\/]*(([^/\\\\]*?)(\.([^\.\\\\/]+?)|))[\\\\/\.]*$%im', $path, $pathinfo)) {
            if (array_key_exists(1, $pathinfo)) {
                $ret['dirname'] = $pathinfo[1];
            }
            if (array_key_exists(2, $pathinfo)) {
                $ret['basename'] = $pathinfo[2];
            }
            if (array_key_exists(5, $pathinfo)) {
                $ret['extension'] = $pathinfo[5];
            }
            if (array_key_exists(3, $pathinfo)) {
                $ret['filename'] = $pathinfo[3];
            }
        }
        switch ($options) {
            case PATHINFO_DIRNAME:
            case 'dirname':
                return $ret['dirname'];
            case PATHINFO_BASENAME:
            case 'basename':
                return $ret['basename'];
            case PATHINFO_EXTENSION:
            case 'extension':
                return $ret['extension'];
            case PATHINFO_FILENAME:
            case 'filename':
                return $ret['filename'];
            default:
                return $ret;
        }
    }


    //! Strip newlines to prevent header injection.
    /*!
     * @param string $str
     * @return string
     */
    public function secureHeader($str) {
        return trim(str_replace(array("\r", "\n"), '', $str));
    }

    
    //! Normalize line breaks in a string.
    
    /*!
     * Converts UNIX LF, Mac CR and Windows CRLF line breaks into a single line break format.
     * Defaults to CRLF (for message bodies) and preserves consecutive breaks.
     * @param string $text
     * @param string $breaktype What kind of line break to use, defaults to CRLF
     * @return string
     * @static
     */
    public static function normalizeBreaks($text, $breaktype = "\r\n") {
        return preg_replace('/(\r\n|\r|\n)/ms', $breaktype, $text);
    }

    //! Detect if a string contains a line longer than the maximum line length allowed.
    /*!
     * @param string $str
     * @return bool
     * @static
     */
    public static function hasLineLongerThanMax($str) {
        //+2 to include CRLF line break for a 1000 total
        return (boolean)preg_match('/^(.{'.(self::MAX_LINE_LENGTH + 2).',})/m', $str);
    }

    //! Allows for public read access to 'to' property.
    /*!
     * @note: Before the send() call, queued addresses (i.e. with IDN) are not yet included.
     * @return array
     */
    public function getToAddresses() {
        return $this->_to;
    }

    //! Allows for public read access to 'cc' property.
    /*!
     * @note: Before the send() call, queued addresses (i.e. with IDN) are not yet included.
     * @return array
     */
    public function getCcAddresses() {
        return $this->_cc;
    }

    //! Allows for public read access to 'bcc' property.
    /*!
     * @note: Before the send() call, queued addresses (i.e. with IDN) are not yet included.
     * @return array
     */
    public function getBccAddresses() {
        return $this->_bcc;
    }

    //! Allows for public read access to 'ReplyTo' property.
    /*!
     * @note: Before the send() call, queued addresses (i.e. with IDN) are not yet included.
     * @return array
     */
    public function getReplyToAddresses() {
        return $this->_replyTo;
    }

    //! Allows for public read access to 'all_recipients' property.
    /*!
     * @note: Before the send() call, queued addresses (i.e. with IDN) are not yet included.
     * @return array
     */
    public function getAllRecipientAddresses() {
        return $this->_allRecipients;
    }

    //! Set mail priority 
    /*!
     * @param mixed $priorty null (default), 1 = High, 3 = Normal, 5 = low 
     *
     * @return bool TRUE if successfully set the priority
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setPriority($priority = NULL) {
        if (!isset($priority) && 
            $priority != 1 &&
            $priority != 3 &&
            $priority != 5) {
            return FALSE;
        }
        $this->_priority = $priority;
        return TRUE;
    }

    //! Get mail priority
    /*
     * @return mixed priority
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getPriority() {
        return $this->_priority;
    }


    //! Set if keeping smtp connection alive
    /*!
     * @param bool $keepAlive keep the connection alive or not
     *
     * @return bool TRUE if successfully set the variable
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setSMTPKeepAlive($keepAlive) {
        if (!isset($keepAlive) || !WizardValidator::isBool($keepAlive)) {
            return FALSE;
        }
        $this->_SMTPKeepAlive = $keepAlive;
        return TRUE;
    }

    //! Get SMTPKeepAlive
    /*
     * @return bool SMTPKeepAlive
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getSMTPKeepAlive() {
        return $this->_SMTPKeepAlive;
    }

    //! Set confirmatin email address
    /*!
     * @param string $keepAlive keep the connection alive or not
     *
     * @return bool TRUE if successfully set the helo
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setConfirmReadingTo($confirmReadingTo) {
        if (!isset($confirmReadingTo) || !WizardValidator::isEmail($confirmReadingTo)) {
            return FALSE;
        }
        $this->_confirmReadingTo = $confirmReadingTo;
        return TRUE;
    }

    //! Get confirmation email address
    /*!
     * @return string confirmReadingTo
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getConfirmReadingTo() {
        return $this->_confirmReadingTo;
    }

    //! Set pretend to send email
    /*!
     * @param bool $pretend to send email or not
     *
     * @return bool TRUE if successfully set the pretend
     *              FALSE otherwise
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function setPretend($pretend) {
        if (!isset($pretend) || !WizardValidator::isBool($pretend)) {
            return FALSE;
        }
        $this->_pretend = $pretend;
        return TRUE;
    }

    //! Get pertend to send email
    /*!
     * @return bool pretend
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public function getPretend() {
        return $this->_pretend;
    }

    //! Set cid_type to send email
    /*!
     * @param bool $cid_type to send email or not
     *
     * @return bool TRUE if successfully set the cid_type
     *              FALSE otherwise
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function setCidType($cid_type) {
        if (!isset($cid_type) || !WizardValidator::isStringNotEmpty($cid_type) || strlen($cid_type) > 16 || !ctype_alnum($cid_type)) {
            return FALSE;
        }
        $this->_cid_type = $cid_type;
        return TRUE;
    }

    //! Get CID TYPE
    /*!
     * @return bool cid_type
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getCidType() {
        return $this->_cid_type;
    }

    //! Set cid to send email
    /*!
     * @param bool $cid to send email or not
     *
     * @return bool TRUE if successfully set the cid
     *              FALSE otherwise
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function setCid($cid) {
        if (!isset($cid) || !WizardValidator::isStringNotEmpty($cid) || strlen($cid) > 128 || !ctype_print($cid)) {
            return FALSE;
        }
        $this->_cid = $cid;
        return TRUE;
    }

    //! Get CID
    /*!
     * @return bool cid
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getCid() {
        return $this->_cid;
    }

    //! Set authMethod to send email
    /*!
     * @param bool $authMethod to send email or not
     *
     * @return bool TRUE if successfully set the authMethod
     *              FALSE otherwise
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function setAuthMethod($authMethod) {
        if (!isset($authMethod) || !WizardValidator::isStringNotEmpty($authMethod) 
            || in_array($authMethod, array('CRAM-MD5', 'LOGIN', 'PLAIN'))) {
            $this->logMessage("Auth method implemented by Wizard SMTP: 'CRAM-MD5', 'LOGIN', 'PLAIN'", self::DEBUG_LOWLEVEL);
            return FALSE;
        }
        $this->_authMethod = $auth_method;
        return TRUE;
    }

    //! Get auth method
    /*!
     * @return bool auth_method
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getAuthMethod() {
        return $this->_authMethod;
    }
}

