<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2010-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Anonymous
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: class.LMNotificationsRemoteRepository.php 528 2014-09-29 21:36:00Z shaun $
 */

require_once('class.LMNotificationsRepository.php');
require_once('class.LMNotificationsEntry.php');
require_once('class.WebRequest.php');
require_once('class.WebResponse.php');

class LMNotificationsRemoteRepository extends LMNotificationsRepository {

    //! version of the format we are expecting back
    static $version = 1;

    private $uri = null;
    private $user = null;
    private $pass = null;

    public function __construct($uri, $product, $user = null, $pass = null) {
        parent::__construct($product);
        if (!$uri) {
            throw new Exception('Invalid URI defined.');
        }
        $this->uri = $uri;
        $this->user = $user;
        $this->pass = $pass;
    }

    //! download entries since the given timestamp and load them into our object
    /*!
     * @param numeric $timestamp The unixtime to retrieve entries since.
     *
     * @return void
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * TODO: convert to return and indicate failure?
     */
    protected function load($timestamp) {
        if (!is_numeric($timestamp)) {
            // TODO: log/raise error?
            return;
        }
        $timestampURI = rawurlencode($timestamp);
        $productURI = rawurlencode($this->product);
        $versionURI = rawurlencode(self::$version);
        $req = new WebRequest("{$this->uri}?timestamp={$timestampURI}&feed_id={$productURI}&feed_version={$versionURI}", WEBREQUEST_GET);
        $req->set_auth($this->user, $this->pass);
        // WebRequest::send() throws exceptions.
        try {
            $resp = $req->send();
        } catch (Exception $e) {
            // TODO: log somehow? or raise error?
            return;
        }
        if ($resp->get_code() != 200) {
            // TODO: log somehow? or raise error?
            return;
        }
        $entries = $this->parse($resp->data);
        foreach ($entries as $entry) {
            if ($entry->timestamp > $timestamp) {
                $this->addEntry($entry);
            }
        }
    }

    //! parse XML feed data.
    /*!
     * @param string $data XML string to parse
     *
     * @return array of LMNotificationsEntry objects
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    protected function parse($data) {
        if (!is_string($data) || strlen($data) === 0) {
            // TODO: log/raise error?
            return array();
        }
        $xml = @simplexml_load_string($data);
        if ($xml === FALSE) {
            // TODO: log/raise error?
            return array();
        }
        $entries = array();
        foreach ($xml->posts->entry as $node) {
            $entry = new LMNotificationsEntry;
            // TODO: could we cast to integer?
            $entry->timestamp = (string)$node->timestamp;
            $entry->title = (string)$node->title;
            $entry->author = (string)$node->author;
            $entry->severity = (string)$node->severity;
            $entry->short_description = (string)$node->short_description;
            $entry->description = (string)$node->description;
            $entry->type = (string)$node->type;
            $entry->from_remote = TRUE;

            $entries[] = $entry;
        }
        return $entries;
    }
}
