<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2010-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Anonymous
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: class.LMNotificationsRepository.php 1174 2016-04-28 17:29:37Z shaun $
 */

abstract class LMNotificationsRepository {
    protected $product = null;

    /** Array of entries where the key is the unix timestamp of the entry */
    protected $entries = null;

    public function __construct($product) {
        if (!$product) {
            throw new Exception('Invalid product defined.');
        }
        $this->product = $product;
    }

    public function getMostRecent($num = 1) {
        $entries = $this->getEntries();
        // sort by key, descending.
        krsort($entries, SORT_NUMERIC);
        reset($entries);
        return current($entries);
    }

    //! Retrieve any entries which are newer than $timestamp.
    /*!
     * @param mixed $timestamp The timestamp to retrieve entries back to.
     *   If NULL/non-numeric, we will default to all in the last month.
     *
     * @param array $alwaysLoad optional array of entry types to always load
     *                          regardless of the associated timestamp
     *
     * @return Associative array of timestamp => LMNotificationsEntry objects.
     *   These will be sorted.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getEntries($timestamp = NULL, $alwaysLoad = NULL) {
        // TODO: could use some optimizations here. for now, we are just going to
        // regenerate the entries array with new data
        if (!is_numeric($timestamp)) {
            // if timestamp not specified, grab anything in the last month
            $timestamp = time() - ( 60 * 60 * 24 * 30 );
        }
        $this->entries = array();
        $this->load($timestamp, $alwaysLoad);

        // sort by key, descending.
        krsort($this->entries, SORT_NUMERIC);
        reset($this->entries);
        return $this->entries;
    }

    //! add a new LMNotificationsEntry object to our entry list.
    /*!
     * @return void
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    protected function addEntry($entry) {
        if (!is_a($entry, 'LMNotificationsEntry')) {
            // TODO: log/raise error?
            return;
        }
        $this->entries[$entry->timestamp] = $entry;
    }

    protected abstract function load($timestamp);
}
