<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id$
 */

require_once('lib.control.php');
require_once('class.LMNotifications.php');


class MMTipsTricks extends LMNotifications {

    protected static $feedId = 15;

    protected static $numUnreadEntries = NULL;


    public function __construct() {
        global $config;

        $uri = 'https://notifications.linuxmagic.com';
        if ($config && array_key_exists('notification-uri', $config)) {
            $uri = $config['notification-uri'];
        }

        parent::__construct(static::$feedId, $uri, "/home/shared/alerts");
    }



    /**
    * Returns the number of tips that were added after the passed in timestamp
    *
    * @param int a unix timestamp of the last time the tips were viewed
    *
    * @return int The number of unread tips
    *
    * @author Logan Borean <loganb@linuxmagic.com>
    */
    public function getNumUnread($lastTimeViewTips) {
        if (static::$numUnreadEntries !== NULL) {
            return static::$numUnreadEntries;
        } 

        $allEntries = $this->getAllEntries();

        $numUnreadTips = 0;
        foreach ($allEntries as $tipEntry) {
            $tipPublishTime = $tipEntry->timestamp;
            if ($tipPublishTime < $lastTimeViewTips) {
                break;
            }
            $numUnreadTips++;
        }

        static::$numUnreadEntries = $numUnreadTips;
        return $numUnreadTips;
    }

}



