<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: bmsListsModule.php 10044 2019-07-19 15:52:23Z deiony $
 */
require_once('lib.module.php');
require_once('lib.datatable.php');

class bmsListsModule extends Module {
    //! internal array of all available BMS lists.
    private $lists;
    //! expected array label used for rendered radio buttons
    private $radio_name;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
        $this->lists = array(
            '4' => array(
                    'description' => 'UCEPROTECT-1',
                    'help' => 'bmsUCE1'),
            '5' => array(
                    'description' => 'UCEPROTECT-2',
                    'help' => 'bmsUCE2'),
            '13' => array(
                    'description' => 'PSBL',
                    'help' => 'bmsPSBL'),
            '23' => array(
                    'description' => 'SORBS-DUL',
                    'help' => 'bmsSORBS'),
            '35' => array(
                    'description' => 'MIPSpace-all (includes MIPSpace-worst, poor, and pros)',
                    'help' => 'bmsMIPSall'),
            '40' => array(
                    'description' => 'MIPSpace-worst',
                    'help' => 'bmsMIPSworst'),
            '41' => array(
                    'description' => 'MIPSpace-poor',
                    'help' => 'bmsMIPSpoor'),
            '42' => array(
                    'description' => 'MIPSpace-pros',
                    'help' => 'bmsMIPSpros'),
            '36' => array(
                    'description' => 'RATS-Dyna',
                    'help' => 'bmsRatsDyna'),
            '37' => array(
                    'description' => 'RATS-NOPTR',
                    'help' => 'bmsRatsNoptr'),
            '38' => array(
                    'description' => 'RATS-Spam',
                    'help' => 'bmsRatsSpam'),
        );
        $this->radio_name = "bmslist";
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $recommended_bms, $output_only;
        // init variables
        $selected_lists = "";
        $selected_arr = array();
        $flagarr = array();
        $ret = TRUE;

       	if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }	

        if ($action == 'save_lists') {
            if (isset($_POST[$this->radio_name])) {
                foreach ($_POST[$this->radio_name] as $key => $value) {
                    // set rule as enable if is enable or if is flag
                    if ($value == "true" || $value == "flag") {
                        $selected_arr[] = $key;
                    }
                    // add the flag rules to array
                    if ($value == "flag") {
                        $flagarr[] = $key;
                    } 
                }
            }
            $selected_lists = implode("\n", $selected_arr);
            if (!strlen($selected_lists)) {
                $ret = clearControl(BMS_ENABLED);
            } else {
                $ret = setControl(BMS_ENABLED, $selected_lists);
            }

            // let write the flags
            $flagRet = TRUE;
            // if PRO write the flag rules
            if (isProfessional() && defined('BMS_FLAG')) {
                $flagstr = implode("\n", $flagarr);
                if (!strlen($flagstr)) {
                    $flagRet = clearControl(BMS_FLAG);
                } else {
                    $flagRet = setControl(BMS_FLAG, $flagstr);
                }
            }
            
            if ($ret === TRUE && $flagRet === TRUE) {
                $notices[] = intl("bmsLists"). " " . intl("saved");
            // NOTE: regarding localizing errors, it may be worth considering having the setControl stuff
            //       return common 'label' strings - with the i18n values residing in the top level locale.
            } else { // TODO: localize the error from functions
                // let look where we have the errors from 
                if ($ret !== TRUE) {
                    $errors[] = $ret;
                }
                if ($flagRet !== TRUE) {
                    $errors[] = $flagRet;
                }
            }
        } else if ($action == "set_bmslist_to_defaults") {
            $flagRet = TRUE;

            if ((isProfessional() || (isset($isFrontend) && $isFrontend)) && defined('BMS_FLAG')) {
                $flagRet = clearControl(BMS_FLAG);
            }
            if (sizeof($recommended_bms) == 0) {
                $ret = clearControl(BMS_ENABLED);
            } else {
                $ret = setControl(BMS_ENABLED, implode("\n", $recommended_bms));
            }
            if ($ret === TRUE && $flagRet === TRUE) {
                $notices[] = intl("bmsLists"). " " . intl("saved");
            } else {
                if ($ret != TRUE) {
                    $errors[] = $ret;
                }
                if ($flagRet != TRUE) {
                    $errors[] = $flagRet;
                }
            }
        } elseif ($action === "lookup_ip") {
        
            $output_only = 1;

            $ipAddress = isset($_POST['ip_address']) ? trim($_POST['ip_address']) : NULL;
            // init the result array
            $data = array(
                'status'    => FALSE,
                'message'   => '',
                'data'      => array(),
            );
            // validate the IP address
            if (filter_var($ipAddress, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) === FALSE) {
                $data['message'] = intl('ipAddressInvalid');
                print json_encode($data);
                return;
            }

            // init the variables that are passed by reference to exec
            $output = '';
            $return = '-2';

            // build the command
            $command = BMS_CHECK . ' -i '  . escapeshellarg($ipAddress);
            exec($command, $output, $return);

            switch ($return) {

                // no hits 
                case '0':
                    $data['status'] = TRUE;
                    $data['message'] = '<span class="text-red">' . intl('ipNotListed') . '</span>';
                    $data['status'] = TRUE;
                break;
                // listed 
                case '1':
                    if (isset($output[0]) && !empty($output[0])) {
                        $data['message'] =  '<span class="text-green">' . intl('ipListed', 2) . '</span>';
                        $data['status'] = TRUE;
                        $data['data'] = explode(',', $output[0]);
                    } else {
                        $data['message'] = intl('bmsCheckProblem');
                    }
                break;
                // unexpected exit code
                case '-1':
                default :
                    $data['message'] = intl('bmsCheckProblem');
                break;
            }
            // send  JSON to frontend
            print json_encode($data);
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module, $recommended_bms, $controller, $isFrontend;
        // add help blurbs to js to be used on each list 
        foreach ($this->lists as $rule=>&$data) {
            $this->_i18n_js[] = $data['help'];
        }

        // set up the table style
        $table = new magicspamDataTable();
        $table->setClass("row", array("leftColumn", "centerColumn", "rightColumn"));
        $table->setClass("headerRow", array("leftColumn", "centerColumn", "rightColumn"));
        $table->setHeader(array(intl('bmsListName'), '&nbsp;', intl('status')));

        // load the enabled items from rules.enabled
        // array will be empty if file is missing or empty
        $lists_enabled = array();
        $lists_flag = array();

        $tmp = getControl(BMS_ENABLED, "");
        if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
            $lists_enabled = explode("\n", $tmp);
        }

        if (isProfessional() && defined('BMS_FLAG')) { 
            $tmp = getControl(BMS_FLAG, "");
            if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
                $lists_flag = explode("\n", $tmp);
            }
        }

        // set the default and enabled value for each rule
        foreach ($this->lists as $rule => &$data) {
            $data['default'] = in_array($rule, $recommended_bms);
            $data['enabled'] = in_array($rule, $lists_enabled);
        }

        // and... determine if these are already enabled at the 'global' level.
        if (  (isset($controller) && $controller == "magicspam.live.php") 
           || (isset($isFrontend) && $isFrontend) ) {

            $masterEnabled = array();
            // grab the 'master' server level enabled rules
            $tmp = getControl(BMS_ENABLED, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterEnabled = explode("\n", $tmp);
            }

            $masterFlag = array();
            // grab the 'master' server level Flag rules
            $tmp = getControl(BMS_FLAG, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterFlag = explode("\n", $tmp);
            }

            // and now we lock these out from being edited
            // to go only higher than the master control
            foreach ($this->lists as $rule => &$data) {
                // if the rule is flag on master the we set bymaster 1, frontend user can change to ON only
                // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to flag or ON

                if (in_array($rule, $masterEnabled) && in_array($rule, $masterFlag)) {
                    $data['bymaster'] = 1;
                } elseif (in_array($rule, $masterEnabled)) {
                    $data['bymaster'] = 2;
                } else {
                    $data['bymaster'] = 0;
                }
            }
        }
        $listlabel = intl('bmsListLabel');
        foreach ($this->lists as $rule => &$data) {
            $help = isset($data['help']) ? $data['help'] : "";
            $desc = htmlspecialchars(isset($data['description']) ? $data['description'] : "");
            if (!isset($data['bymaster'])) {
                $data['bymaster'] = 0;
            }
            $internal = "({$listlabel}: " . htmlspecialchars($rule) . ")";
            $target = "_blank";
            $desc = renderListDescription($desc, $internal, NULL, "MS.showHelpPopup(this,'X','".$help."'); return false;", $target);
            

            $isEnabled = FALSE;
            if (isset($data['enabled'])) {
                $isEnabled = $data['enabled'];
            }

            // a flag option will be enabled and flag
            if ($isEnabled && in_array($rule, $lists_flag)) {
                $isEnabled = 'flag';
            }

            switch ($data['bymaster']) {
                // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                case 2: 
                    $isEnabled = TRUE;
                    $defstat = generateLabel("enforced");
                break;

                // if the rule is flag on master the we set bymaster 1, frontend user can change to ON only
                case 1:
                    if ($isEnabled !== TRUE) {
                        $isEnabled = 'flag';
                    }
                    if (isset($isFrontend) && $isFrontend) {
                        $defstat = generateLabel("restricted"); 
                    } else {

                        $defstat = generateLabel("recommended", $data['default']); 
                    }
                break;
            
                // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to flag or ON
                default: 
                    $defstat = generateLabel("recommended", $data['default']);
                    break; 
            }
            
            $isLocked = 0;
            if (isset($data['bymaster'])) {
                $isLocked = $data['bymaster'];
            }

            // for PRO show the 3 state slider
            if (isProfessional()) {
                $sliderWidget = renderSliderThree($isEnabled, $isLocked, $this->radio_name . "[{$rule}]", 'bmsListsForm');
            } else {
                $sliderWidget = renderSlider($isEnabled, $isLocked === 0 ? FALSE : TRUE, $this->radio_name . "[{$rule}]", 'bmsListsForm');
            }

            $table->addRow(array($desc, $defstat, $sliderWidget));
        }

        $this->_i18n_js[] = 'on';
        $this->_i18n_js[] = 'off';
        $this->_i18n_js[] = 'ipAddressInvalid';
        
        // Render only BMS information for freemium users
        if (isFreemium()) {
            renderTemplate('bmsListsFreemium', array(
                'blurb' => intl('bmsBlurbFree'),
            ));
        } else {
            renderTemplate('bmsLists', array(
                'ipLabel' => intl('ipAddressLabel'),
                'ipPlaceholder' => intl('ipPlaceholder'),
                'lookupLabel' => intl('lookup'),
                'blurb' => intl('bmsBlurb'),
                'table' => $table->getRenderString(),
                'module' => $module,
                'save' => intl('save'),
                "defaults" => intl("defaults"),
                "url" => defined("FE_INDEX_PREFIX") ? FE_INDEX_PREFIX : (defined("INDEX_PREFIX") ? INDEX_PREFIX : ''),
            ));
        }
    }

    //! method to retrieve the title of the current module
    /*!
     * @returns string  the title for the current module.
     */
    public function getModuleTitle() {
        return intl('bmsLists');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('bms.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array("set-action.js", "bms-lists.js");
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // retrieve and store the enabled and flag only BMS lists
        $configurations[BMS_ENABLED] = getControl(BMS_ENABLED, "", MASTERCONTROLDIR);
        if (isProfessional() && defined('BMS_FLAG')) {
            $configurations[BMS_FLAG] = getControl(BMS_FLAG, "", MASTERCONTROLDIR);
        }

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // store the enabled and flag only BMS lists
        setControl(BMS_ENABLED, $configurations[BMS_ENABLED]);
        if (isProfessional() && defined('BMS_FLAG')) {
            if (array_key_exists(BMS_FLAG, $configurations)
                && WV::isStringNotEmpty($configurations[BMS_FLAG])) {
                setControl(BMS_FLAG, $configurations[BMS_FLAG]);
            } else {
                clearControl(BMS_FLAG);
            }
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
