<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: feedbackModule.php 10395 2019-09-16 18:11:15Z deiony $
 */
require_once('lib.module.php');

require_once('wizard/lib.wizard_validator.php');
require_once('wizard/lib.wizard_debug.php');
require_once('wizard/lib.wizard_smtp.php');
require_once('wizard/lib.wizard_mail.php');
require_once('wizard/lib.http_var.php');

class feedbackModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if ($action == "send") {
            // retrieve the rating, sender and comments input
            $rating = HttpVar::getPost("rating", NULL);
            $sender = HttpVar::getPost("sender", NULL);
            $comments = HttpVar::getPost("comments", '');

            // validate the rating and sender, the comments are optional
            if (!WV::isIntPositiveOrZero($rating)) {
                $errors[] = intl('feedback_rating_failure');
                return;
            }
            if (!WV::isEmail($sender)) {
                $errors[] = intl('feedback_sender_failure');
                return;
            }

            // prepare the data for binding with the email template
            $bind = array(
                "package" => MAGICSPAM_PACKAGE,
                "rating" => $rating,
                "comments" => $comments,
            );

            // build the email body
            $body = getTemplate("feedbackEmail", $bind, FALSE);
            if (!WV::isStringNotEmpty($body)) {
                $errors[] = intl('feedback_email_body_failure');
                return;
            }

            // initialize the mailer to connect to our email server
            $mailer =  new WizardMail();
            $mailer->setHost('mail.cityemail.com');

            // set the helo
            // NOTE: by default, the helo can be set to the server IP address or
            //       localhost which is at odds with our email server spam rules
            $mailer->setHelo('feedback-mailer.magicspam.com');

            // set the sender and recipient
            $mailer->setSender($sender);
            $mailer->addTO('feedback@magicspam.com');

            // set auto-generated and auto-response headers
            $mailer->addCustomHeader("Auto-Submitted", "auto-generated");
            $mailer->addCustomHeader("X-Auto-Response-Suppress", "OOF");

            // set the email subject and body
            $mailer->setSubject("MagicSpam Feedback");
            $mailer->setBody($body);

            // send the feature request email
            if (!$mailer->send()) {
                $errors[] = intl('feedback_send_failure') . ": " . $mailer->getError()['detail'];
            } else {
                $notices[] = intl('feedback_send_success');
            }
        }
    }

    //! method to render the full page module.
    public function renderModule() {
        global $module;
        $this->_i18n_js[] = 'feedback_no_rating_failure';
        renderTemplate('feedback', array(
            'module' => $module,
            'admin_email' => getControl('admin_email', 'Unknown'),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array('feedback.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array('feedback.js');
    }
}
