<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *
 * $Id: glanceModule.php 5053 2015-08-19 15:50:22Z shaun $
 */
require_once('lib.module.php');
require_once('lib.rcptcounter.php');
require_once('lib.datatable.php');

class glanceModule extends Module {

    //! array object to hold the 'counts' of SPAM, HAM, EXEMPT
    private $counts;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function __construct($l) {
        global $errors;
        parent::__construct($l, FALSE);
        $this->counts = NULL;
        // load the stats for this month.
        $stats = new RCPTCounter(date('n'));
        // NOTE: we don't check for errors because there may be no stats yet!
        if ($stats->load()) {
            $statsDate = $stats->get_date(date('j'));
            if ($statsDate !== FALSE) {
                $statsYear = date('Y', $statsDate);
                if ($statsYear == date('Y')) {
                    // from that stack, get the data for 'today' (NOTE: RCPT_TYPE_ALL defined in rcptcounter class file)
                    $this->counts = $stats->get_value(date('j'), RCPT_TYPE_ALL);
                }
            }
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // nothing to process here!
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function renderModule() {
        global $module, $license, $admin_email;
        // grab the counts
        $spam = $this->getCount('spam');
        $ham = $this->getCount('ham');
        $exempt = $this->getCount('exempt');
        $ratelimit = $this->getCount('ratelimit');
        $total = $spam + $ham + $exempt + $ratelimit;
        $enabled = getControl('enabled', '0');
        
        // prepare our labels
        $prefix = intl('glanceRcpts') . ' ';
        $header = intl('glanceHeader');

        // set up our 'dynamic' table class.
        $tstable = new magicspamDataTable;
        $tstable->addRow( array($prefix . intl('glanceReceived'), $total));
        $tstable->addRow( array($prefix . intl('glanceHam'), $ham));
        $tstable->addRow( array($prefix . intl('glanceSpam'), $spam));
        $tstable->addRow( array($prefix . intl('glanceExempt'), $exempt));
        $tstable->addRow( array($prefix . intl('glanceLimit'), $ratelimit));

        // set up our system summary table
        $stable = new magicspamDataTable;

        // we start with the assumption that this is a 'freemium' model of MagicSpam
        $version = MAGICSPAM_VERSION;

        $licensekey = 'MSPLSK-FREE';
        $expires = intl('never');
        // now.. we find out if there is a license key installed
        if (!isFreemium()) {
            $licensekey = $license->getCode();
            $licenseType = intl($license->getSKUType());
            if ($license->isExpired()) {
                $expires = intl('expired');
            } else {
                $this->_i18n_js[] = 'expiry_blurb';
                $expires = "<a class=\"expiryLink\" onclick=\"MS.showHelpPopup(this, 'X', 'expiry_blurb');\" title=\"" . intl('expiry_blurb') . "\">" . $license->getExpiry() . "</a>";
            }
            if (strlen($licensekey) === 0) {
                $licensekey = intl('notset');
            }
        } else {
            // XXX: intl?
            $version .= " Free Edition";
        }

        $stable->addRow(array(intl('license'), $licensekey));
        $stable->addRow(array(intl('version'), $version));
        $upgrade = NULL;
        $upgradeBlurb = NULL;

        $this->_i18n_js[] = 'glanceLicenseCloseLabel';
        $this->_i18n_js[] = 'glanceLicenseBlurb';

        if (!isFreemium()) {
            if (!$license->isNew()) {
                $stable->addRow(array(intl('expires'), $expires));
                $stable->addRow(array(intl('licenseType'), $licenseType));
                $stable->addRow(array(intl('glanceEmail'), $admin_email));
            } else {
                $stable->addBlurbRow('<a href="?module=settings" onclick="doFormSubmit({\'hiddenFields\':{\'module\':\'settings\',\'page\':\'system_settings\'}}); return false;">' . intl('enter_license') . '</a>');
            }
        } else {
            $stable->addBlurbRow('<a href="#" onclick="MS.showHelpPopup(this, '
                . '\'glanceLicenseCloseLabel\', \'glanceLicenseBlurb\'); return false;">'
                . intl("whyupgradeblurb") . '</a>');
        }
        // and, we add a blurb in the event protection is disabled
        $blurb = '';

        if ($enabled == "0") {
            $blurb .= '<div id="glanceNotEnabled">' . "\n";
            $blurb .= intl('glanceNotEnabled');
            $blurb .= '</div><!-- end #glance_not_enabled -->' . "\n";
        }

        renderTemplate('glance', array(
                                       'module' => $module,
                                       'header' => $header,
                                       'blurb' => $blurb,
                                       'tstable' => $tstable->getRenderString(),
                                       'stable' => $stable->getRenderString(),
                                       'upgrade' => $upgrade,
                                       'upgradeBlurb' => $upgradeBlurb,
                                      ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function getCSS() {
        return array('glance.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function getJS() {
        return array();
    }

    //! internal utility function to get the count of a given type.
    /*!
     * @param   string  the 'type' index.
     *
     * @returns integer the count found or '0' if nothing found.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    protected function getCount($type) {
        if (!is_string($type) || strlen($type) == 0) {
            return 0;
        }
        if (is_array($this->counts) &&
            isset($this->counts[$type])) {
            return $this->counts[$type];
        }
        if ($type === 'ratelimit') {
            $count = isset($this->counts['ip_ratelimit']) ? $this->counts['ip_ratelimit'] : 0;
            $count += isset($this->counts['user_ratelimit']) ? $this->counts['user_ratelimit'] : 0;
            return $count;
        }
        return 0;
    }
}
