<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Kyle Macdonald <kylem@linuxmagic.com>
 *            Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: ipReputationModule.php 10603 2019-11-12 18:08:41Z deiony $
 */
require_once("lib.module.php");

class ipReputationModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array (
            'bmsLists',
            'rblLists',
        );
        // set module to be migratable
        $this->migratable = TRUE;
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array_merge(array('form-submit.js', 'ajax.js'), $this->subModuleJS());
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Kyle Macdonald <kylem@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;
        $firstExpanded = FALSE;
        $submodule = (isset($_POST['submodule']) && strlen($_POST['submodule'])) ? $_POST['submodule'] : NULL;

        // print out hte header and blurb
        // XXX: this whole module is a mess.. should be using templates instead of inline echo!
        echo "<h2>" . intl('ip_reputation') . "</h2>";
        echo "<div class=\"masterBlurb\"><p>" . intl('ip_blurb') . "</p></div>";

        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }
            $activeAddon = "";
            $isActive = FALSE;
            $img = 'expand-icon.png';

            // Assemble the name of the session variable
            $collapseState = $m . "__collapse";
            
            // Validation to make certain we got a value
            if (isset($_SESSION[$collapseState]) && $_SESSION[$collapseState] != '') {
                $collapseString = $_SESSION[$collapseState];
            
            // Else open by default
            } else {
                $collapseString = 1;
            }
   
            // If 1 then we set it to be active
            if ($collapseString == 1) {
                $isActive = TRUE;
            }

            // Define the session variable here
            $_SESSION[$collapseState] = $collapseString;

            // Alters the style based if the tab is active
            if ($isActive) {
                $activeAddon = " collapsibleModuleActive";
                $img = 'collapse-icon.png';
                $firstExpanded = TRUE;
            }

            // XXX: inline echo is EVIL
            echo "<div class=\"collapseHeader\" onclick=\"toggleSectionAndUpdateState(this, '{$m}','{$collapseString}','" . IMAGEPATH . "');\">\n";
            echo "  <div class=\"collapseLeft\"></div>\n";
            echo "  <div class=\"collapseCenter\">\n";
            echo "    <span class=\"collapseTitle\">" . $modules[$m]->getModuleTitle() . "</span>\n";
            echo "    <span class=\"collapseActions\"><img src=\"" . IMAGEPATH . "{$img}\"/></a></span>\n";
            echo "  </div>\n";
            echo "  <div class=\"collapseRight\"></div>\n";
            echo "</div><!-- end collapseHeader-->\n";
            echo "<div id=\"{$m}\" class=\"collapsibleModule{$activeAddon}\">\n";

            $modules[$m]->renderModule();

            echo "</div><!-- end collapsibleModule -->\n";
        }
    }
}
