<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: settingsModule.php 6968 2017-09-26 21:32:58Z shaun $
 */
require_once("lib.module.php");

class logModule extends Module
{
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);
        $this->submodules = array(
            'logSearch',
            'logTable',
        );

        // these settings are only meant for the MS PRO series
        if (isProfessional()) {
            array_splice($this->submodules, 1, 0, "authLogSearch");
        }

        // Attempts to load each of the submodules
        global $modules;
        foreach ($this->submodules as $m) {
            if (array_key_exists($m, $modules)) {
                continue;
            }
            loadModule($m);
        }
    }

    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])
                && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    public function renderModule() {
        global $modules;

        // render the menu
        renderMenu();

        // render the title and overview
        echo "<h2>" . intl("logSearchTitle") . "</h2>"
           . "<div class=\"descBuffer\">"
           .     intl("logSearchOverview")
           . "</div>";

        // render the submodules
        foreach ($this->submodules as $m) {
            $modules[$m]->renderModule();
        }
    }

    //! method to return the title of the page to renderHeader
    public function getPageTitle() {
        return intl('log');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    public function getJS() {
        return $this->subModuleJS();
    }
}
