<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * This is the primary working interface page for the WHM module.
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id$
 */
require_once('lib.module.php');

// set up a default path for configuration file unless otherwise 
// defined in optional suite_config.php.
if (!defined('CONFIG_FILE')) {
    define('CONFIG_FILE' ,'/etc/magicspam/control/config');
}

class loginModule extends Module {
    //! internal private representation of the login name
    private $admin_login = NULL;
    
    //! internal private representation of the login password
    // NOTE: this will be a 'crypt' format password.
    private $admin_pass = NULL;

    //! internal error message buffer.
    private $lasterror = "";

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        // part of loading this module, we want to 
        // load the configuration for login as well.
        $config = $this->loadConfig(CONFIG_FILE);
        if (is_array($config)) {
            if (isset($config['admin_login'])) {
                $this->admin_login = $config['admin_login'];
            }
            if (isset($config['admin_password'])) {
                $this->admin_pass = $config['admin_password'];
            }
        }
        // this module does not render on the menu.
        parent::__construct($l, FALSE);

    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $authenticated;
        // if we were asked to validate the login...
        if ($action == 'login') {
            // make sure we have the necessary arguments
            if (  isset($_POST['mmc_login']) && strlen($_POST['mmc_login'])
               && isset($_POST['mmc_password']) && strlen($_POST['mmc_password'])
               && !is_null($this->admin_login) && !is_null($this->admin_pass)) {
                // test against the set variables
                $pwd = trim($_POST['mmc_password']);
                $login = strtolower(trim($_POST['mmc_login']));
                if (  $login  == $this->admin_login
                   && crypt($pwd, $pwd) == $this->admin_pass) {
                    // and if good, set the session variables
                    $_SESSION['mmc_auth_username'] = $login;
                    $_SESSION['mmc_auth_password'] = $pwd;
                    $authenticated = TRUE;
                    $notices[] = intl('login_success');
                } else {
                    $errors[] = intl('login_invalid');
                }
            } else {
                $errors[] = intl('login_invalid');
            }
        // if we were asked to write out the new config...
        } else if ($action == 'set_password') {
            // make sure we have the necessary arguments
            if (isset($_POST['mmc_new_password']) && strlen($_POST['mmc_new_password']) >= 5) {
                // validate the arguments
                $config = array(
                                 'admin_login' => 'admin'
                               , 'admin_password' => crypt($_POST['mmc_new_password'], $_POST['mmc_new_password'])
                               );

                // write out the config file
                if ($this->writeConfig(CONFIG_FILE, $config)) {
                    // and set the session XXX: do we want this?
                    $_SESSION['mmc_auth_username'] = 'admin';
                    $_SESSION['mmc_auth_password'] = $config['admin_password'];
                    $authenticated = TRUE;
                    $notices[] = intl('login_password_saved');
                } else {
                    $errors[] = $this->lasterror;
                }
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        // if we are missing configuration items
        if (is_null($this->admin_login) || is_null($this->admin_pass)) {
            // render the 'first run' password set form.
            renderTemplate('loginSetup', array(
                                 'title' => intl('login_welcome_title')
                                ,'blurb' => intl('login_welcome_blurb')
                                ,'login_password_field' => intl('login_password_field')
                                ,'login_confirm_password' => intl('login_password_confirm')
                                ,'save' => intl('save')
                                ,'password_mismatch' => intl('login_password_mismatch')
                                ,'password_length' => intl('login_password_length')
                                ));
        } else if (!self::isAuthenticated()) {
            // if we aren't logged in
            // display the login form
            renderTemplate('login', array(
                                 'title' => intl('login_welcome_title')
                                ,'blurb' => intl('login_blurb')
                                ,'login_name_field' => intl('login_name')
                                ,'login_password' => intl('login_password')
                                ,'login' => intl('login_login')
                                ));
        } else {
            // we should never be trying to render this module if we are authenticated... someone is trying something fishy...
        }

    }

    public function getPageTitle() {
        return 'MagicSpam Log In';
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('login.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('login.js');
    }

    //! static public method for determining if the current session is authenticated
    /*!
     * @returns boolean true if authenticated, false if not.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    static public function isAuthenticated() {
        // not sure if superglobals need to be declared like this?
        global $_SESSION;
        // NOTE: we *assume* the calling program has already started a session (session_start()).

        // check for the presence of our session cookies...
        if (   isset($_SESSION['mmc_auth_username'])
            && strlen($_SESSION['mmc_auth_username'])
            && isset($_SESSION['mmc_auth_password'])
            && strlen($_SESSION['mmc_auth_password'])) {
            return TRUE;
        }
        return FALSE;
    }

    //! internal utility function for loading the configuration file
    /*!
     * @param   string  the configuration file
     *
     * @returns mixed   FALSE on fail, array on success.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     *
     * SIDE EFFECT: error messages are stored in internal error message buffer.
     */
    private function loadConfig($cfg_file) {
        global $_SESSION;
        $config = array();
        // sanity check
        if (!is_string($cfg_file) || !strlen($cfg_file)) {
            $this->lasterror = intl('login_invalid_config');
            return FALSE;
        }
        // make sure the file exists
        if (!file_exists($cfg_file)) {
            $this->lasterror = intl('login_missing_config');
            return FALSE;
        }
        // read the file into line array
        if (! $data = file($cfg_file)) {
            $this->lasterror = intl('login_config_open_error');
            return FALSE;
        }
        // parse the lines
        foreach ($data as $line) {
            if (preg_match("/^(\S+)\s*:\s*(\S+)\s*$/", trim($line), $matches)) {
                $label = trim($matches[1]);
                $val = trim($matches[2]);
                $config[$label] = $val;
            }
        }
        // and return our findings.
        return $config;
    }

    //! internal utility function for writing out the configuration file.
    /*!
     * @param   string  the configuration file to write to
     * @param   array   the configuration set to write out.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    private function writeConfig($cfg_file, $config) {
        // validate the parameters
        if ( !is_string($cfg_file) || !strlen($cfg_file)
          || !is_array($config) || !sizeof($config)) {
            $this->lasterror = intl('login_invalid_config');
            return FALSE;
        }

        $fh = fopen($cfg_file,"w");
        if (!$fh) {
            $this->lasterror = intl('login_config_open_error');
            return FALSE;
        }

        foreach ($config as $key => $val) {
            fprintf($fh,"%s: %s\n",$key,$val);
        }
        fclose($fh);
        return TRUE;
    }
}

?>
