<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id:
 */
require_once('lib.module.php');
require_once('lib.replayutility.php');

class replayModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $output_only;
        if ($action === 'replay' && $_POST['output_only'] === '1') {
            $output_only = 1;
            $this->handleReplayAction();
        }
    }

    //! handles a replay action by calling the replay utility
    protected function handleReplayAction() {
        // call the replay utility
        $replayCmd = ReplayUtility::getInstance();
        if ($replayCmd->callReplayUtility($_POST) === FALSE) {
            $error_html = $this->build_error_html($replayCmd->error());
            print $error_html;
            return;
        }

        // render and populate a popup with the result
        $res = $replayCmd->getResult();

        // build and return the html to the AJAX call
        $html = $this->build_html($res, $_POST['ip'], $_POST['from']);
        print $html;
    }

    //! builds the html for the replay results popup
    /*!
     * @param    array    the contents of the AJAX call
     * @param    string   the sender address for the mail session
     * @param    string   the ip address for the mail session
     * 
     * @returns  string   the html for the replay results popup
     */
    protected function build_html($data, $ip, $sender) {
        $html = '';

        // start off by adding the title
        $html .= '<div id="replay-results">' . "\n";
        $html .= '    <h3>Replay results</h3>' . "\n\n";

        // check whether there were any hits
        if (count($data['bms_hits']) === 0 && count($data['spam_hits']) === 0) {
            // an informational blurb informing user that there was no hit
            $html .= '    <p class="replay-blurb">' . intl('replay_blurb_no_hit') . '</p>' . "\n";
            // option to block the IP
            $html .= '    <a class="replay-block" ';
            $html .= '       href="#" ';
            $html .= '       onclick="logSearchAddIPToList(\'' . $ip . '\', \'black\', \'' . intl('ls_alert_blacklist') . '\');"';
            $html .= '>' . intl('ls_block_IP') . ': [' . $ip . ']</a></br>' . "\n";
            // option to block the Sender
            $html .= '    <a class="replay-block" ';
            $html .= '       href="#" ';
            $html .= '       onclick="logSearchAddSenderToList(\'' . $sender . '\', \'black\', \'' . intl('ls_alert_blacklist') . '\');"';
            $html .= '>' . intl('ls_block_Sender') . ': [' . $sender . ']</a>' . "\n";

        } else {
            // an informational blurb informing user that there was at least one hit
            $html .= '    <p class="replay-blurb">' . intl('replay_blurb_hit') . '</p>' . "\n\n";

            // report on every spam and bms hits
            foreach ($data as $list => $hits) {
                // display the list name
                $html .= '    <h4>' . intl($list) . '</h4>' . "\n";

                // special case when there is no hit
                if (count($hits) === 0) {
                    $html .= '    <span class="replay-rule">None</span>' . "\n";
                }

                // display each hit in the list
                foreach($hits as $num => $hit) {
                    $html .= '    <span class="helplink" onclick="MS.showHelpPopup(this, \'X\', \'' . intl('help_replay_' . $hit) .'\'); return false;"></span>';
                    $html .= '    <span class="replay-rule">' . intl('replay_' .$hit) . '</br></span>' . "\n";
                }

                $html .= "\n";
            }
        }

        // a separate div for the close button
        $html .= "\n";
        $html .= '    <div class="replay-bottom">' . "\n";
        $html .= '        <button type="button" onclick="hideReplay();">Close</button>' . "\n";
        $html .= '    </div>' . "\n";
        $html .= '</div>';
        return $html;
    }

    protected function build_error_html($error) {
        // set AJAX status response to 400 code
        header("HTTP/1.0 400 error");

        // start building the error html
        $html = '';
        $html .= '<ul>' . "\n";
        $html .= '    <li>' . $error . '</li>' . "\n";
        $html .= '</ul>';
        return $html;
    }

    //! method to render the module
    public function renderModule() {
        return;
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return $this->subModuleJS();
    }

}
