<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: exemptionModule.php 8748 2018-10-30 16:22:04Z deiony $
 */
require_once("lib.module.php");

class securitySettingsModule extends Module {
    //! class constructor
    public function __construct($l) {
        parent::__construct($l, TRUE);
        $this->submodules = array(
            'countryAuth',
        );

        // set module to be migratable
        $this->migratable = TRUE;

        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    public function getPageTitle() {
        return intl('securitySettings'); 
    }

    //! method to process any actions and react accordingly.
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if (isset($subModule) && isset($modules[$subModule]) && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module
    public function renderModule() {
        global $modules;

        // render the menu as this is a top level module
        renderMenu();

        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }
            $modules[$m]->renderModule();
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    public function getCSS() {
        return array_merge(array('list-forms.css', 'user-settings.css'), $this->subModuleCSS());
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    public function getJS() {
        return array_merge(array('ajax.js'), $this->subModuleJS());
    }
}
