<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: serverPoliciesModule.php 9538 2019-03-14 23:30:28Z deiony $
 */
require_once('spamSettingsModule.php');
require_once('lib.rejections.php');

class serverPoliciesModule extends spamSettingsModule {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
        // XXX: consider limiting the serverwide subset .. but for now we'll offer all.
    }

    public function getJS() {
        return array_merge(array('help-popup.js'), parent::getJS());
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // retrieve and store the custom rejection messages
        $rejections = array();
        foreach (array_keys($this->rules) as $rule) {
            $rejection_file = REJECTIONS . $rule;
            $rejection = "";
            if (file_exists($rejection_file)) {
                $rejection = file_get_contents($rejection_file);
            }
            $rejections[$rule] = $rejection;
        }
        $configurations["rejections"] = $rejections;

        // retrieve and store the enabled and flag only spam rules
        $configurations[RULES_ENABLED] = getControl(RULES_ENABLED, "", MASTERCONTROLDIR);
        if (isProfessional() && defined('RULES_FLAG')) {
            $configurations[RULES_FLAG] = getControl(RULES_FLAG, "", MASTERCONTROLDIR);
        }

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // store the custom rejection messages
        $rejection_manager = RejectionsManager::getInstance();
        foreach ($configurations["rejections"] as $rule => $rejection) {
            if (!strlen($rejection)) {
                $rejection_manager->removeRejection($rule);
            } else {
                $rejection_manager->setRejection($rule, $rejection);
            }
        }

        // store the enabled and flag only spam rules
        setControl(RULES_ENABLED, $configurations[RULES_ENABLED]);
        if (isProfessional() && defined('RULES_FLAG')) {
            if (array_key_exists(RULES_FLAG, $configurations)
                && WV::isStringNotEmpty($configurations[RULES_FLAG])) {
                setControl(RULES_FLAG, $configurations[RULES_FLAG]);
            } else {
                clearControl(RULES_FLAG);
            }
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
