<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: serviceStatusModule.php 6173 2017-01-25 17:24:37Z shaun $
 *
 * This is the base definition for all top level modules in MagicSpam.
 * This class must never be used directly but rather be extended to define a
 * working whole.
 */
require_once('lib.module.php');

class serviceStatusModule extends Module {

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if ($action == "start_service") {
            if (  defined('SECURE_BIN') && is_executable(SECURE_BIN)
               && defined('SVC_START') && is_executable(SVC_START)) {
                exec(SECURE_BIN . " " . SVC_START, $output, $rv);
                if ($rv == 0) {
                    $notices[] = intl('start_success');
                } else {
                    $errors[] = intl('start_fail');
                    if (is_array($output)) {
                        foreach ($output as $o) {
                            $errors[] = $o;
                        }
                    }
                    $errors[] = intl('contact_support');
                }
            }
        }
    }

    //! method to render the full page module.
    public function renderModule() {
        // this does nothing.  this is a processing utility module only.
    }

    //! method to determine if a service is not running
    /*!
     * @returns boolean true if all services are running, otherwise false.
     *
     * NOTE: exposed as PUBLIC as global index hooks need access
     *       as well as the protectionKey module to ascertain what to render.
     */
    public function isRunning() {
        if (  defined('SECURE_BIN') && is_executable(SECURE_BIN)
           && defined('SVC_CHECK') && is_executable(SVC_CHECK)) {
            exec(SECURE_BIN . ' ' . SVC_CHECK, $output, $rv);
            if ($rv == 0) {
                return TRUE;
            } else {
                return FALSE;
            }
        }
        // fall through: no service check utility defined, no way to know
        //               also if there were errors calling the script - again no way to
        //               know so no need to freak people out about it.
        return TRUE;
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array();
    }

}
