<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *            Logan Borean <loganb@linuxmagic.com>
 *
 * $Id:
 */
require_once('lib.basestatsmodule.php');
require_once('lib.spamassassinstatsutility.php');

class spamAssassinStatsModule extends BaseStatsModule {

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @author  Logan Borean<loganb@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
    }

    //! Method to render the full page for ?module=stats
    /*!
     * @brief   Assembles our "Daily Statistics" page 
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        // Pass i18n labels to JS for SA stats graph
        $this->_i18n_js[] = 'totalProcessedLabelJS';
        $this->_i18n_js[] = 'totalTriggeredLabelJS';
        $this->_i18n_js[] = 'totalEvadedLabelJS';

        // retrieve the number of top LM rules to retrieve, default is 5
        $numSelections = $this->allowedNumSelections[0];
        if ($this->isValidFormSubmission()) {
            $numSelections = $_POST['num-selections'];
        }

        // call the SA stats utility and retrieve JSON encoded stats buffer
        $buffer = '';
        $saStatsCmd = SpamAssassinStatsUtility::getInstance();
        if ($saStatsCmd->callSpamAssassinStatsUtility(1, $numSelections, $buffer) === TRUE) {
            // decode the json encoded stats buffer
            $decode = json_decode($buffer, TRUE);
            if (!$this->__valid_stats($decode)) {
                echo "<div id=\"errors\">";
                echo "    <ul><li>" . intl('errorDecodeStats') . "</li></ul>";
                echo "</div>";
                return;
            }
        } else {
            echo "<div id=\"notices\">";
            echo "    <ul><li>" . intl('noticeNoStats') . "</li></ul>";
            echo "</div>";
        }
        // Renders the template
        renderTemplate('SAstats', array(
                                         'module' => $module,
                                         'submodule' => 'spamAssassinStats',
                                         'numSelections' => $numSelections,
                                         'totalProcessed' => $decode['TOTAL_MAIL_PROCESSED'] ?: 0,
                                         'totalTriggered' => $decode['TOTAL_MAIL_TRIGGERED'] ?: 0,
                                         'totalEvaded' => $decode['TOTAL_MAIL_EVADED'] ?: 0,
                                         'topRules' => $decode['TOP_RULES'] ?: array(),
                                         'expandImage' => IMAGEPATH . 'expand-icon.png',
                                        ));
    }

    //! ensures that the stats from the SpamAssassin stats utility are valid
    /*!
     * @brief   Validates the stats from the SpamAssassin stats utility
     *
     * @param   array   the stats from the SpamAssassin stats utility
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    private function __valid_stats($stats)
    {
        $expected_stats = array('TOTAL_MAIL_PROCESSED', 'TOTAL_MAIL_TRIGGERED', 'TOTAL_MAIL_EVADED', 'TOP_RULES'); 

        // sanity check
        if (empty($stats) || !is_array($stats)) {
            return FALSE;
        }

        // ensure that all the expected stats are present
        $actual_stats = array_keys($stats);
        foreach ($expected_stats as $stat) {
            if (!in_array($stat, $actual_stats, TRUE)) {
                return FALSE;
            }
        }

        return TRUE;
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array("SA-stats.css");
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        global $module;

        // default to including the Chart JS
        $js = array("Chart.bundle.min.js");

        // we only want to load the JS when this specific submodule is selected
        if ($module == 'stats' && isset($_POST['page']) && $_POST['page'] == 'spamAssassinStats') { 

            // the new versions of Plesk do not need the Chart JS
            if (class_exists("pm_ProductInfo")) {
                $version = (int)pm_ProductInfo::getVersion();
                if ($version >= 17) {
                    $js = array();
                }
            }

            // do not forget to include the base SA Stats JS
            $js[] = "SA-stats.js";
        }

        return $js;
    }
}
