<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: spamSettingsModule.php 10044 2019-07-19 15:52:23Z deiony $
 */
require_once('functions.php');
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('lib.rejections.php');

class spamSettingsModule extends Module {
    protected $rules;
    protected $rules_subset;

    //! string. this is to track the request action that was last run.
    // why do we have it? because for 'set recommended defaults' we want to also
    // be able to show, from javascript, a popup bubble warning. this is a way
    // to communicate with javascript to tell it which action was run.
    private $_action_flag = '';

    private $_flag_rules = array();

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);

        $this->rules = array(
            'block_lists' => array(
                'description' => 'Block messages based on source IP Reputation (BMS&trade;)',
                'help' => 'sp_ipRep',
                'bymaster' => TRUE,
                'enabled' => TRUE,
                'freemium' => FALSE,
            ), 'block_rbl_lists' => array(
                'description' => 'Block messages based on source IP Reputation (RBL)',
                'url' => "http://www.linuxmagic.com/power_of_ip_reputation.html",
                'help' => 'sp_ipRepRBL',
                'bymaster' => TRUE,
                'enabled' => TRUE,
                'freemium' => FALSE,
            ), 'block_ip_in_addr' => array(
                'description' => 'Block addresses without domain (e.g. johnny@192.168.1.1)',
                'help' => 'sp_noDomain',
            ), 'check_dynamic_reverse_dns_default' => array(
                'description' => 'Block Mail Servers using COMMON Dynamic/DUL IP space (DYNA REGEX COMMON)',
                'help' => 'sp_dynaRegex',
            ), 'check_dynamic_reverse_dns' => array(
                'description' => 'Block Mail Servers using Probable Dynamic/DUL IP space (DYNA REGEX FULL) ',
                'description_freemium' => 'Block Mail Servers using Probable Dynamic/DUL IP space (DYNA REGEX LITE) ',
                'help' => 'sp_dynaRegex',
                'freemium' => TRUE,
            ), 'check_confirmed_dyna' => array(
                'description' => 'Block Mail Servers using Confirmed Dynamic/DUL IP space (DYNA REGEX) ',
                'help' => 'sp_dynaRegex',
                'freemium' => FALSE,
            ), "check_ip_reverse_dns" => array(
                "description" => "Require Mail Servers to have rDNS configured",
                "help" => 'sp_revDns',
            ), 'check_reverse_dns_list' => array(
                'description' => 'Block Mail Servers reported as Spam Source (NONDUL REGEX) ',
                'help' => 'sp_dnsList',
                'freemium' => FALSE,
            ), "require_helo" => array(
                "description" => "Sending Server must identify itself (HELO)",
                "help" => 'sp_reqHelo',
            ), "valid_helo_domain" => array(
                "description" => "Server Identification should be sane (FQDN HELO) ",
                "help" => 'sp_validHelo',
            ), "resolve_helo_domain" => array(
                "description" => "Confirm Server Identification Resolves (HELO) ",
                "help" => 'sp_resolveHelo',
            ), "rfc_mail_from" => array(
                "description" => "MAIL FROM: must meet RFC2822 specifications",
                "help" => 'sp_rfcFrom',
            ), "require_full_addr" => array(
                "description" => "Strict Email Address Parsing (RFC Compliance)",
                "help" => 'sp_fullAdd',
            ), "valid_from_domain" => array(
                "description" => "Valid FROM domain (A or MX Record)",
                "help" => 'sp_validFrom',
            ), "valid_ptr" => array(
                "description" => "PTR record should be FQDN (Best Practices) ",
                "help" => 'sp_validPtr',
            ), "known_sender_forgery" => array(
                "description" => "&quot;Known Sender Forgery&quot; (KSF) &reg; Block phishing attempts",
                "help" => 'sp_KSF',
                'freemium' => FALSE,
            ),"block_nonauth_ipv6" => array(
                "description" => "Block mail from IPv6 addresses that are not authenticated",
                "help"      => 'sp_IPV6',
                'freemium'  => TRUE,
            ),
        );
        // if this is freemium, we simply hide the block_lists and rbl_lists rule as we don't
        // support custom rejections in freemium mode.
        if (isFreemium()) {
            array_shift($this->rules);
            array_shift($this->rules);
        }
    }

    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    protected function setPoliciesSubset($subset) {
        if (isset($subset) && is_array($subset)) {
            $this->rules_subset = $subset;
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $recommended_rules, $recommended_rules_free;

        // store the action - we want to provide it to the javascript
        // environment in some cases.
        $this->_action_flag = $action;
        // sets the state of the collaspible modules
        if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }

        // action to update what rules are enabled.
        if ($action == 'save_server_policies') {
            $rulesarr = array();
            $flagarr = array();
            $rulesstr = "";
            // we don't want to save these rules here
            $ignoreRules = array(
                'block_lists',
                'block_rbl_lists',
            );

            foreach ($_POST["rules"] as $key=>$value) {
                if (in_array($key, $ignoreRules)) {
                    continue;
                }
                // set rule as enable if is enable or if is flag
                if ($value == "true" || $value == "flag") {
                    $rulesarr[] = $key;
                }
                if ($value == "flag") {
                    $flagarr[] = $key;
                }
            }
            $rulesstr = implode("\n", $rulesarr);           
            if (!strlen($rulesstr)) {
                $ret = clearControl(RULES_ENABLED);
                
            } else {
                $ret = setControl(RULES_ENABLED, $rulesstr);

            }

            // let write the flags
            $flagRet = TRUE;
            // if PRO write the flag rules
            if (isProfessional() && defined('RULES_FLAG')) {
                $flagstr = implode("\n", $flagarr);
                if (!strlen($flagstr)) {
                    $flagRet = clearControl(RULES_FLAG);
                } else {
                    $flagRet = setControl(RULES_FLAG, $flagstr);
                }
            }

            if ($ret === TRUE && $flagRet === TRUE) {
                $notices[] = intl("serverPolicies"). " " . intl("saved");
            } else { // TODO: localize the error from functions
                if ($ret !== TRUE) {
                    $errors[] = $ret;
                }
                if ($flagRet !== TRUE) {
                    $errors[] = $flagRet;
                }
            }

        // action to set rules to the defaults.
        } else if ($action == 'set_server_policies_to_defaults') {
            $retFlag = TRUE;
            if (isFreemium()) {
                $ret = setControl(RULES_ENABLED, implode("\n", $recommended_rules_free));
            } else {
                $ret = setControl(RULES_ENABLED, implode("\n", $recommended_rules));

                if (defined('RULES_FLAG')) {
                    $retFlag = clearControl(RULES_FLAG);
                }
            }

            if ($ret === TRUE && $retFlag === TRUE) {
                $notices[] = intl("serverPolicies"). " " . intl("saved");
            } elseif (!$ret) {
                $errors[] = $ret;
            } elseif (!$retFlag) {
                $errors[] = $retFlag;
            }

        // action to load the current custom rejection. this is used through
        // an ajax request right now only it appears.
        } else if ($action == 'load_rejection') {
            if (isset($_REQUEST['rule']) && strlen($_REQUEST['rule'])) {
                // XXX: this is to be modified once rejection utility supports
                //   loading.
                if (file_exists(REJECTIONS . $_REQUEST['rule'])) {
                    $contents = file_get_contents(REJECTIONS
                        . $_REQUEST['rule']);
                    if ($contents !== FALSE) {
                        echo $contents;
                    }
                    // TODO: what if it was false?
                }
            }

        // action to update a custom rejection message.
        } else if ($action == 'save_rejection') {
            // we only permit this action if custom rejections are actually
            // enabled and allowed.
            if (!$this->_customRejectionsAllowed()) {
                $errors[] = intl('serverPoliciesNoRejections');
                return;
            }
            if (isset($_REQUEST['notice']) && isset($_REQUEST['rule'])) {
                $msg = trim($_REQUEST['notice']);
                $msg = str_replace("\n", " ", $msg);
                $msg = str_replace("\r", " ", $msg);

                // grab a handle on the singleton
                $rmanager = RejectionsManager::getInstance();

                if (!strlen($msg)) {
                    if (!$rmanager->removeRejection($_REQUEST['rule'])) {
                        $errors[] = $rmanager->error();
                    } else {
                        $notices[] = intl('saved');
                    }
                } else {
                    if (!$rmanager->setRejection($_REQUEST['rule'], $msg)) {
                        $errors[]  = $rmanager->error();
                    } else {
                        $notices[] = intl('saved');
                    }
                }
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        $enabled_rules = array();

        // adding warning labels so they are available in js for the KSF warning dialog
        $this->_i18n_js[] = 'ksfCloseLabel';
        $this->_i18n_js[] = 'ksfWarningMessage';

        // and help popups
        foreach ($this->rules as $rule => &$data) {
            $this->_i18n_js[] = $data['help'];
        }

        // load the enabled items from rules.enabled
        $tmp = getControl(RULES_ENABLED, "");
        if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
            $enabled_rules = explode("\n", $tmp);
        }

        // set the default and enabled values
        $this->_setDefaultEnabled($enabled_rules);

        $this->_getFlagRules();

        // set up the base column classes for rendering
        $rowClasses = array('ruleDesc', 'recommendation', 'ruleStatus');

        // now we also need to determine if we are rendering rejection messages
        // as well as merging 'master' rules
        $renderRejections = FALSE;
        $showRejectionsLink = '';
        $showRejections = FALSE;
        if (isset($_POST['showRejections']) && $_POST['showRejections'] === '1') {
            $showRejections = TRUE;
        }
        $header = '<span class="ruleDesc">' . intl('serverPoliciesRule') . "</span>\n"
                . '<span class="recommendation">&nbsp;</span>' . "\n"
                . '<span class="ruleStatus">' . intl('status') . "</span>\n"
                . '<span class="ruleActions">' . intl('rejections') . "</span>\n";

        // if this is not a 'client' frontend
        if (!$this->_setLockedRules()) {
            // and if custom rejections are allowed (freemium/not freemium)
            if ($this->_customRejectionsAllowed()) {
                // then we want to allow custom rejections
                $renderRejections = TRUE;
                // and render a toggle link
                $showRejectionsLink = '<span class="rejectionLink"><input type="checkbox" id="toggleBox" onclick="toggleRejections(this);"';
                if ($showRejections) {
                    $showRejectionsLink .= ' checked';
                }
                $showRejectionsLink .= '> '
                                    . intl('show_rejections')
                                    . '</span>' . "\n";
                // now, we also modify the classes if we are showing the rejection buttons per request.
                if ($showRejections) {
                    $rowClasses = array('ruleDescSmall',
                                        'recommendationSmall',
                                        'ruleStatus',
                                        'ruleActionsVisible');
                    // and set up the header we want rendered
                    $header = '<span class="ruleDescSmall">' . intl('serverPoliciesRule') . "</span>\n"
                            . '<span class="recommendationSmall">&nbsp;</span>' . "\n"
                            . '<span class="ruleStatus">' . intl('status') . "</span>\n"
                            . '<span class="ruleActionsVisible">' . intl('rejections') . "</span>\n";
                } else {
                    $rowClasses[] = 'ruleActions';
                }
            }
        }
            
        // Validate when we determine the state of the tab
        if (isset($_SESSION['serverPolicies__collapse']) && $_SESSION['serverPolicies__collapse'] != '') {
            $collapseState = $_SESSION['serverPolicies__collapse'];
        
        // Else open by default
        } else {
            $collapseState = 1;
        }

        // prepare our table
        $table = $this->_prepareDataTable($rowClasses, $renderRejections);

        renderTemplate('serverPolicies', array(
            'mainHeader'            => intl("serverPolicies"),
            'blurb'                 => intl("serverPoliciesBlurb"),
            'table'                 => $table->getRenderString(),
            'module'                => $module,
            'save'                  => intl("save"),
            'defaults'              => intl("defaults"),
            'header'                => $header,
            'showRejectionsLink'    => $showRejectionsLink,
            'rejectionEditor'       => intl('rejectionEditor'),
            'rejectionInstructions' => intl('rejectionInstructions'),
            'charsLeft'             => intl('charsLeft'),
            'image_path'            => IMAGEPATH,
            'actionFlag'            => htmlspecialchars($this->_action_flag),
            'collapseState'         => $collapseState,
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getCSS() {
        return array('server-policies.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array(
                     'ajax.js'
                     ,'server-policies.js'
                    );
    }

    //! determine whether custom rejections are usable.
    /*!
     * @returns bool True if you can use custom rejections right now, or false
     *   if not.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * by default we do not show the custom rejections column/options.
     * you can enable it through a system setting.
     *
     * as well, we don't offer this feature in freemium currently, so on top of
     * the setting we also do not allow this if we are currently in freemium
     * mode.
     */
    private function _customRejectionsAllowed() {
        // in freemium mode we don't allow custom rejections.
        if (isFreemium()) {
            return FALSE;
        }
        // we're allowed to use custom rejections!
        return TRUE;
    }


    //! internal method to prep the 'default' rule status
    /*!
     * @param   array   the array of rules that are currently enabled.
     */
    private function _setDefaultEnabled($enabled_rules) {
        global $recommended_rules, $recommended_rules_free;
        // safety check to ensure we are working with an array.
        if (!is_array($enabled_rules)) {
            $enabled_rules = array();
        }
        // for each rule that is in the list
        foreach ($this->rules as $rule => &$data) {
            // set the 'default' allocation based on if its freemium mode or licensed.
            if (isFreemium()) {
                $data['default'] = in_array($rule, $recommended_rules_free);
            } else {
                $data['default'] = in_array($rule, $recommended_rules);
            }
            // then, set the 'enabled' status based on the passed in array.
            $data['enabled'] = in_array($rule, $enabled_rules);
        }
        // done - all other information pass through is held in internal variable '$rules'
    }

    //! internal method for determining any 'locked out' rules based on where this is being run.
    /*!
     * @returns boolean TRUE if this is a 'per user' frontend that has locked rules
     *                  FALSE if this is the administrator interface and thus unlocked.
     */
    private function _setLockedRules() {
        // we need access to a global variable for cases of MS PRO
        global $controller, $isFrontend;
        // we zero in on the current PRO designation for client frontend
        if (  (isset($controller) && $controller == 'magicspam.live.php') 
           || (isset($isFrontend) && $isFrontend) ) {
            $masterEnabled = array();
            // grab the 'master' server level enabled rules
            $tmp = getControl(RULES_ENABLED, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterEnabled = explode("\n", $tmp);
            }

            $masterFlag = array();
            // grab the 'master' server level Flag rules
            $tmp = getControl(RULES_FLAG, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterFlag = explode("\n", $tmp);
            }

            // and now we lock these out from being edited
            // to go only higher than the master control
            foreach ($this->rules as $rule => &$data) {
                // if the rule is flag on master the we set bymaster 1, frontend user can change to ON only
                // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to flag or ON
                if (in_array($rule, $masterEnabled) && in_array($rule, $masterFlag)) {
                    $data['bymaster'] = 1;
                } elseif (in_array($rule, $masterEnabled)) {
                    $data['bymaster'] = 2;
                } else {
                    $data['bymaster'] = 0;
                }
            }
            return TRUE;
        }
        return FALSE;
    }

    //! internal method to prepare a data table for rendering
    /*!
     * @param array $columnClasses TODO document
     * @param bool $renderRejections TODO document
     *
     * @return magicspamDataTable
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    private function _prepareDataTable($columnClasses, $renderRejections) {
        global $isFrontend;
        $defstat = "";

        // safety check
        if (!is_array($columnClasses)) {
            $columnClasses = array();
        }
        if (!is_bool($renderRejections)) {
            $renderRejections = FALSE;
        }
        // create our table object
        $table = new magicspamDataTable();

        // set the classes
        // XXX: what happens if its empty?
        $table->setClass("row", $columnClasses);
    	$first = 0;
        // and generate our data rows
        foreach ($this->rules as $rule => &$data) {
            if (  !isset($this->rules_subset)
               || (isset($this->rules_subset) && is_array($this->rules_subset))) {

                $help = isset($data['help']) ? $data['help'] : "";
                $description = isset($data['description']) ? $data['description'] : "";
                if (!isset($data['bymaster'])) {
                    $data['bymaster'] = FALSE;
                }

                if (isFreemium() && isset($data['description_freemium'])) {
                    $description = $data['description_freemium'];
                }

                $internal = htmlspecialchars(intl('rule_name')) . ": " . htmlspecialchars($rule);
                // some rules have note text under certain conditions.
                $noteHTML = NULL;
                // dynaregex: if we're in freemium mode then note about needing
                // a license to get updates.
                if (isFreemium() && $rule === 'check_dynamic_reverse_dns') {
                    $noteHTML = '<p>';
                    $noteHTML .= intl('serverPoliciesDynaFreemiumUpdate');
                    $noteHTML .= '</p>';
                } else if (!isFreemium() && $rule === 'check_dynamic_reverse_dns') {
                    $noteHTML = '<p>';
                    $noteHTML .= intl('serverPoliciesLastUpdate');
                    $noteHTML .= ' ';
                    $noteHTML .= htmlspecialchars(
                        $this->_getRegexUpdateTime('dyna'));
                    $noteHTML .= '</p>';
                }
                if (!isFreemium() && $rule === 'check_confirmed_dyna') {
                    $noteHTML = '<p>';
                    $noteHTML .= intl('serverPoliciesLastUpdate');
                    $noteHTML .= ' ';
                    $noteHTML .= htmlspecialchars(
                        $this->_getRegexUpdatetime('confirmed'));
                    $noteHTML .= '</p>';
                }
                $desc = renderListDescription($description, $internal, NULL,
                    "MS.showHelpPopup(this,'X', '".$help."'); return false;",
                    "_blank", $noteHTML);

                // determine if this is a freemium disabled rule
                $upgradeRequired = FALSE;

                if (isFreemium() && isset($data['freemium']) && $data['freemium'] === FALSE) {
                    // turn the rule off
                    $data['enabled'] = FALSE;
                    // determine the link id to use
                    $link_id = 'jfM0Tey1pN';
                    if ($rule === 'known_sender_forgery') {
                        $link_id = '4JOCmi5F42';
                    }
                    // set the status as locked
                    $defstat = '<span class="islocked"><a href="http://www.magicspam.com/store.php?link_id=';
                    $defstat .= $link_id;
                    $defstat .= '" target="_blank">' . intl('upgrade_required') . '</a></span>' . "\n";
                    // change anything else that might affect rendering
                    $data['bymaster'] = FALSE;
                    $upgradeRequired = TRUE;
                } else {

                    $ruleEnabled = FALSE;
                    if (isset($data['enabled']) && $data['enabled'] === TRUE) {
                        $ruleEnabled = TRUE;
                    }
                    if ($ruleEnabled === TRUE && in_array($rule, $this->_flag_rules)) {
                        $ruleEnabled = 'flag';
                    }

                    switch ($data['bymaster']) {
                        // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                        case 2: 
                            $ruleEnabled = TRUE;
                            $defstat = generateLabel("enforced");
                        break;

                        // if the rule is flag on master the we set bymaster 1, frontend user can change to ON only
                        case 1: 
                            if ($ruleEnabled !== TRUE) {
                                $ruleEnabled = 'flag';
                            }
                            if (isset($isFrontend) && $isFrontend) {                              
                                $defstat = generateLabel("restricted"); 
                            } else {
                                $defstat = generateLabel("recommended", $data['default']);        
                            }                                                                     
                        break;
                    
                        // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to flag or ON
                        default: 
                    
                            $defstat = generateLabel("recommended", $data['default']);
                            break; 
                    }
                }
                $isLocked = $upgradeRequired;
                if (!$isLocked) {
                    if (isset($data['bymaster']) ) {
                        $isLocked = $data['bymaster'];
                    }
                }
                
                if (isProfessional()) {
                    $statusWidget = renderSliderThree($ruleEnabled, $isLocked, "rules[{$rule}]", 'serverPoliciesForm');
                } else {
                    $statusWidget = renderSlider($ruleEnabled, $isLocked == 0 ? FALSE : TRUE, "rules[{$rule}]", 'serverPoliciesForm');
                }

                // ok so now we can prep our row.
                $rowArr = array($desc, $defstat, $statusWidget);
                if ($renderRejections) {
                    $rejectButton = sprintf("<button class=\"msbutton\" onclick=\"editRule('%s', '%s');return false;\"><span>%s</span></button>\n",
                        $rule,
                        defined("INDEX_PREFIX") ? INDEX_PREFIX : "",
                        file_exists(REJECTIONS . $rule) ? intl('edit') : intl('create')
                    );
                    $rowArr[] = $rejectButton;
                }
                // set an id on the dyna regex rule row.
                $rowID = NULL;
                if ($rule === 'check_dynamic_reverse_dns') {
                    $rowID = 'row_check_dynamic_reverse_dns_freemium';
                } else if ($rule === 'check_confirmed_dyna') {
                    $rowID = 'row_check_confirmed_dyna';
                }
                $table->addRow($rowArr, $rowID);
            }
        }
        return $table;
    }

    //! find the last update time of the dyna regex list.
    /*!
     * @return string Text describing the last update time.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * TODO: does this belong in this class?
     *
     * we open and read the header of the dyna regex file.
     * we parse out the date and if we can.
     * we then translate this into visible text.
     */
    private function _getRegexUpdateTime($prefix) {
        // TODO: can we have a define to just /etc/magicspam?
        //   this is under /etc/magicspam/dynaregex, but
        //   we appear to only have /etc/magicspam/control as a define.
        // sanity check argument
        if (!is_string($prefix) || !strlen($prefix)) {
            return "";
        }
        $isCompiled = TRUE;
        $regexFile = CONTROLDIR . '/../dynaregex/' . $prefix . '.cregexes';
        if (!file_exists($regexFile)) {
            // compiled not found, look for plain text version
            $regexFile = CONTROLDIR . '/../dynaregex/' . $prefix . '.regexes';
            if (!file_exists($regexFile)) {
                return intl('serverPoliciesRegexFileNotFound');
            } else {
                $isCompiled = FALSE;
            }
        }
        if (!is_readable($regexFile)) {
            return intl('serverPoliciesRegexFileNotReadable');
        }
        // if this is a non compiled version, then test the format
        if (!$isCompiled) {
            // first line of the file has what we want.
            // it looks like:
            // # Dynamic Style Regular Expression Listing, Autogenerated by LinuxMagic, October 1, 2014, 11:34 am
            $fh = fopen($regexFile, 'r');
            if (!$fh) {
                return intl('serverPoliciesRegexFileCouldNotOpen');
            }
            $line = fgets($fh);
            fclose($fh);
            if ($line === FALSE) {
                return intl('serverPoliciesRegexFileCouldNotRead');
            }
            $line = trim($line);
            // parse out the part we want.
            $res = preg_match('/Autogenerated by LinuxMagic, (.*)$/', $line,
                $matches);
            if ($res !== 1 || count($matches) !== 2 || strlen($matches[1]) === 0) {
                return intl('serverPoliciesRegexFileCorrupt');
            }
        }

        // ok.. let's stat that thing
        $fstat = stat($regexFile);
        if (!is_array($fstat) || sizeof($fstat) < 10) {
            return intl('serverPoliciesRegexFileNotReadable');
        }

        $unixtime = $fstat[9];

        // NOTE: see lib.logsearch date() call too.
        $timeFormat = @date("l, M d Y @ g:i A", $unixtime);
        if ($timeFormat === FALSE) {
            return intl('serverPoliciesRegexDateFormat');
        }
        return $timeFormat;
    }

    //! internal method to get flag rules 
    /*!
     * This will work only for Pro version
     * on Freemium this array will be empty
     */
    private function _getFlagRules() {

        // in freemium mode we don't allow flag rules
        if (!isProfessional() || !defined('RULES_FLAG')) {
            $this->_flag_rules = array();
            return;
        }

        // load the flagonly items from rules.flagonly
        $tmp = getControl(RULES_FLAG, "");
        // something was returned, explode the rules into an array
        if (strlen($tmp) > 0) {
            $flag_rules = explode("\n", $tmp);
        }
        // make sure that we have acutally something in flag_rules array 
        if (isset($flag_rules) && is_array($flag_rules) && count($flag_rules)) {
            $this->_flag_rules = $flag_rules;
        } else {
            $this->_flag_rules = array();
        }
    }
}
