<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *            Sean West <sean@linuxmagic.com>
 *            Shaun A Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: todayStatsModule.php 10602 2019-11-12 18:07:12Z deiony $
 */
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('lib.logsearch.php');
require_once('lib.rcpthourlycounter.php');

class todayStatsModule extends Module {

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @author         Logan Borean<loganb@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if ($action === 'getHourlyStats') {
            $this->respondWithHourlyStats();
        }
    }

    public function respondWithHourlyStats() {
        $counter = new RCPTHourlyCounter();
        $counter->load();



        $data = array();

        $data['summary']['inbound'] = 0;
        $data['summary']['outbound'] = 0;
        $data['summary']['spam'] = 0;
        $data['summary']['total'] = 0;
        
        for ($c = 23; $c >= 0; $c--) {
            $offsetHour = $c - 1;
            $hour = date('G', strtotime("$offsetHour hours ago"));
            $unx = date('U', strtotime("$offsetHour hours ago"));
            $dayDat = $counter->get_value($hour, RCPT_TYPE_ALL);

            if (is_array($dayDat) && isset($dayDat['timestamp']) && abs($dayDat['timestamp'] - $unx) < 3600 ) {

                $ham = $dayDat['ham'];
                $spam = $dayDat['spam'];
                $inbound = $dayDat['inbound'];
                $outbound = $dayDat['outbound'];

                $curHour = date('ga', $dayDat['timestamp']);
                $data['timestamps'][] = $curHour;
                $data['ham'][] = $ham;
                $data['spam'][] = $spam;
                $data['inbound'][] = $inbound;
                $data['outbound'][] = $outbound;


                $data['summary']['inbound']  += $inbound;
                $data['summary']['outbound'] += $outbound;
                $data['summary']['spam']     += $spam;
                $data['summary']['total']    += ($inbound + $outbound);

            } else {
                $curHour = date('ga', mktime($hour));
                $data['timestamps'][] = $curHour;
                $data['ham'][] = 0;
                $data['spam'][] = 0;
                $data['inbound'][] = 0;
                $data['outbound'][] = 0;
            }

        }
        $data['summary']['average'] = round($data['summary']['total'] / 24);

        // format numbers in the summary stats
        foreach ($data['summary'] as $stat => $value) {
            $stat = number_format($value);
        }

        echo json_encode($data);
        // die because this should only be an ajax request
        die();
    }


    //! Method to render the full page for ?module=stats
    /*!
     * @brief   Assembles our "Daily Statistics" page 
     *
     * @params  none
     *
     * @return  none 
     *
     * @author  Robert Bisewski <robertb@linuxmagic.com>   
     *          Shaun A. Johnson <shaun@linuxmagic.com>
     *          Sean West <sean@linuxmagic.com>
     *          Peter Chung <peterc@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        ini_set('max_execution_time', 300);
        $label      = '';
        $renderMenu = FALSE;
        $moduleType = '';
       
        // Determines the top 5 senders / receivers & sorts them from highest
        // to lowest create a LogSearch object. 
        $newLogsSearch = new LogSearch($label, $renderMenu, $moduleType);
        $aStats        = $newLogsSearch->getTopStats();


        // add the i18n we need in javascript.
        $this->_i18n_js[] = 'numMsg';
        $this->_i18n_js[] = 'hour';
        $this->_i18n_js[] = 'chartTitle';
        $this->_i18n_js[] = 'spam';
        $this->_i18n_js[] = 'inbound';
        $this->_i18n_js[] = 'outbound';
        $this->_i18n_js[] = 'ham';


        // Renders the template
        renderTemplate('dayStats', array(
                                         'module'               => $module,
                                         'header'               => intl('todayStatsHeader'),
                                         'refreshLabel'         => intl('todayStatsRefresh'),
                                         'help'                 => intl('help'),
                                         'totalMessages'        => intl('totalMessages'),
                                         'avgMessages'          => intl('avgMessages'),
                                         'inboundMessages'      => intl('inboundMessages'),
                                         'outboundMessages'     => intl('outboundMessages'),
                                         'spamMessages'         => intl('spamMessages'),
                                         'spamInfo'             => intl('spamInfo'),
                                         'inboundInfo'          => intl('inboundInfo'),
                                         'outboundInfo'         => intl('outboundInfo'),
                                         'hamInfo'              => intl('hamInfo'),
                                        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function getCSS() {
        return array("today-stats.css");
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     *                 Peter Chung <peterc@linuxmagic.com>
     */
    public function getJS() {
        global $module;
        $includeChartJS = TRUE;
        if (class_exists("pm_ProductInfo")) {
            $version = (int)pm_ProductInfo::getVersion();
            if ($version >= 17) {
                $includeChartJS = FALSE;
            }
        }

        if ($module == 'stats' && isset($_POST['page']) && $_POST['page'] == 'todayStats' ) {
            if ($includeChartJS) {
                return array("Chart.bundle.min.js", "ajax.js", "today-stats.js");
            } else {
                return array("ajax.js", "today-stats.js");
            }
        // account for 'todaystats' being the default module if no other page was requested..
        } elseif ($module === 'stats' && !isset($_POST['page']) ) {
            if ($includeChartJS) {
                return array("Chart.bundle.min.js", "ajax.js", "today-stats.js");
            } else {
                return array("ajax.js", "today-stats.js");
            }
        // otherwise, it was another page requested.
        } else {
            return array();
        }
    }
}
